@@ -10,12 +10,13 @@ const crypto = require('crypto');
 const uuid = require('uuid');
 const lodash = require('lodash');
 const request = require('postman-request');
-const grpc = require('@grpc/grpc-js');
+const grpc = require('@postman/grpc-js');
 const grpcReflection = require('@postman/grpc-reflection-js');
 const ProtoLoader = require('@postman/proto-loader');
 const Protobuf = require('@postman/protobufjs');
 const EventChannel = require('../channels/EventChannel');
 const PostmanFs = require('../utils/postmanFs');
+const pathIsInside = require('../utils/pathIsInside');
 
 // Definitions included:
 //  - google/api/annotations.proto
@@ -78,7 +79,7 @@ class GRPCClient {
   }
 
   // Opens a native file selector dialog for selecting proto files.
-  async openProtoSelectorDialog () {
+  async openProtoSelectorDialog (workingDir) {
     if (process.platform === 'darwin') {
       app.focus({ steal: true });
     }
@@ -88,7 +89,7 @@ class GRPCClient {
       properties: ['openFile', 'treatPackageAsDirectory']
     });
 
-    return filePaths[0] || null;
+    return sanitizeFilePath(filePaths[0], workingDir || this.defaultWorkingDir) || null;
   }
 
   // Opens a native file selector dialog for selecting folders.
@@ -105,11 +106,13 @@ class GRPCClient {
   }
 
   // Generates a Protobuf JSON descriptor from local proto files.
-  async loadProtoFromFilename (rootFilename, importPaths = []) {
+  async loadProtoFromFilename (rootFilename, importPaths = [], workingDir) {
     const root = new Protobuf.Root();
     const imports = new Map();
     const files = [];
 
+    rootFilename = (new PostmanFs(workingDir || this.defaultWorkingDir, true)).resolvePathSync(rootFilename);
+
     importPaths = [...importPaths, COMMON_PROTO_FILES];
 
     // Overwrite the default "resolvePath" function to utilize import paths.
@@ -163,7 +166,7 @@ class GRPCClient {
     };
   }
 
-  async loadFromServerReflection ({ url, certs, auth, metadata = [], workingDir, options, proxyOptions }) {
+  async loadFromServerReflection ({ url, certs, auth, metadata = [], workingDir, options, proxyOptions, useExperimentalMapFieldDecoding }) {
     url = urlParse(url);
     const { host } = url;
     setupProxy(host, options?.secureConnection, proxyOptions);
@@ -171,12 +174,15 @@ class GRPCClient {
     const wrappedMetadata = wrapMetadata(metadata);
     const channelOptions = getChannelOptions(url, options);
     const credentials = await generateCredentials(certs, auth, wrappedMetadata, workingDir || this.defaultWorkingDir, options);
-    const client = new grpcReflection.Client(host, credentials, channelOptions, wrappedMetadata);
+    const client = new grpcReflection.Client(host, credentials, channelOptions, wrappedMetadata, { useExperimentalMapFieldDecoding });
     const services = await client.listServices();
 
     const descriptors = await Promise.all(
       services
-        .filter((service) => service && service !== 'grpc.reflection.v1alpha.ServerReflection')
+        .filter((service) => service &&
+          service !== 'grpc.reflection.v1alpha.ServerReflection' &&
+          service !== 'grpc.reflection.v1.ServerReflection'
+        )
         .map((service) =>
           client
             .fileContainingSymbol(service)
@@ -356,6 +362,7 @@ class GRPCClient {
 
     const wrappedMetadata = wrapMetadata(metadata);
     const channelOptions = getChannelOptions(url, options);
+    const callOptions = getCallOptions(options);
     const credentials = await generateCredentials(certs, auth, wrappedMetadata, workingDir || this.defaultWorkingDir, options);
     const client = new Service(host, credentials, channelOptions);
     const method = client[location.method];
@@ -364,7 +371,7 @@ class GRPCClient {
       throw new Error(`No gRPC method: ${location.method}`);
     }
 
-    const args = [wrappedMetadata];
+    const args = [wrappedMetadata, callOptions];
     const channel = new EventChannel();
 
     if (!method.requestStream) {
@@ -467,6 +474,18 @@ function getChannelOptions (url, options = {}) {
   };
 }
 
+/**
+ * Generates call options for a gRPC call
+ * @param {Object} options
+ * @param {number} options.timeout - The timeout value in milliseconds.
+ * @returns {grpc.CallOptions}
+ */
+function getCallOptions (options) {
+  const { connectionTimeout } = options;
+
+  return connectionTimeout ? { deadline: new Date(Date.now() + connectionTimeout) } : {};
+}
+
 /**
  * Create channel credentials based on User configuration
  * @param {Object} certs
@@ -914,6 +933,42 @@ function logUnexpectedError (err) {
   }
 }
 
+/**
+ * 1. Convert paths inside working directory to corresponding relative paths, and
+ * 2. Convert win32 file path to posix like path to support cross os file read
+ * Note: Part of it is referenced from src/renderer/runtime-repl/_common/FsHelper.js
+ * @param {String} filePath
+ * @param {String} workingDir
+ *
+ * @returns {String} sanitized filePath
+ *
+ */
+function sanitizeFilePath (filePath, workingDir) {
+  // Set the relative path of the file from the workingDir as the selected file path
+  if (pathIsInside(filePath, workingDir)) {
+    filePath = path.relative(workingDir, filePath);
+  }
+
+  // Ignore sanitization is os platform is not windows
+  if (global.process.platform !== 'win32' || !filePath) {
+    return filePath;
+  }
+
+  // If path is absolute and it's not already sanitized then prepend path with an extra `\\`
+  path.win32.isAbsolute(filePath) && filePath[0] !== '/' && (filePath = `\\${filePath}`);
+
+  // Convert all \\ to /
+  /**
+   * Benchmark for using split.join vs replace regex
+   * The space complexity of split/join is not a big factor
+   *
+   * Split#test x 18,503 ops/sec ±4.30% (76 runs sampled)
+   * Regex#test x 11,308 ops/sec ±7.83% (67 runs sampled)
+   * Fastest is Split#test
+   */
+  return filePath.split(path.win32.sep).join(path.posix.sep);
+}
+
 class PathAwareChannel extends grpc.Channel {
   constructor (target, credentials, options, path) {
     super(target, credentials, options);
