@@ -14,6 +14,7 @@ const {
 } = require('@n1ru4l/push-pull-async-iterable-iterator');
 const EventChannel = require('../channels/EventChannel');
 const PostmanFs = require('../utils/postmanFs');
+const getSystemProxy = require('../../utils/getSystemProxy');
 
 const REQUEST_TIMEOUT = 10000;
 
@@ -93,7 +94,8 @@ async function executeRequest (
     operationType,
     followRedirects = true,
     verifySSLCertificates = false,
-    cwd
+    cwd,
+    proxy: proxyConfig = { system: true, env: true }
   } = options;
 
   const channel = new EventChannel();
@@ -101,9 +103,13 @@ async function executeRequest (
   const authHeaders = await getAuthHeaders(url, auth, graphQLParams);
   const combinedHeaders = assignHeaders(headers, authHeaders);
 
-  const ca = options.ca || options.certificate?.ca ? await loadFile(options.ca || options.certificate?.ca, { cwd }) : undefined;
   const certificate = options.certificate ? await loadCertificate(options.certificate, { cwd }) : {};
 
+  // postman-request: undefined = use environment, false = no proxy
+  const envProxy = proxyConfig.env ? undefined : false;
+  const systemProxy = proxyConfig.system ? await loadSystemProxy(url) : undefined;
+  const proxy = proxyConfig.url ?? systemProxy ?? envProxy;
+
   // Send an HTTP request representing the query or mutation.
   const httpRequest = request.post(
     {
@@ -113,13 +119,12 @@ async function executeRequest (
 
       time: true,
       followRedirect: followRedirects,
+      followAllRedirects: followRedirects,
       strictSSL: verifySSLCertificates,
 
-      ca,
-      cert: certificate?.crt,
-      key: certificate?.key,
-      pfx: certificate?.pfx,
-      passphrase: certificate?.passphrase
+      ...certificate,
+
+      proxy,
     },
     () => {}
   );
@@ -191,15 +196,18 @@ async function executeSubsription (
     legacySubscriptions = false,
     followRedirects = true,
     verifySSLCertificates = false,
+    certificate,
+    cwd
   }
 ) {
   url = validateURL(url, true);
   headers = assignHeaders(headers, await getAuthHeaders(url, auth, graphQLParams));
+  const resolvedCertificates = certificate ? await loadCertificate(certificate, { cwd }) : {};
 
   const channel = new EventChannel();
   const websocketEvents = new EventEmitter();
   const webSocketImpl = createCustomWebSocket(
-    { headers, followRedirects, rejectUnauthorized: verifySSLCertificates },
+    { headers, followRedirects, rejectUnauthorized: verifySSLCertificates, ...resolvedCertificates },
     websocketEvents
   );
   let asyncIterable;
@@ -339,9 +347,7 @@ function getGraphQLResponse (rawResponse) {
     return null;
   }
 
-  if (!isObject(parsed)) return null;
-  if (!isObject(parsed.data) && !Array.isArray(parsed.errors)) return null;
-  return _.pick(parsed, ['data', 'errors', 'extensions']);
+  return isObject(parsed) ? parsed : null;
 }
 
 /**
@@ -403,7 +409,7 @@ const supportedAuthTypes = new Set([
  * Return auth headers based on the auth extension provided by the user.
  *
  * @param {Object} auth - JSON representation of collectionSDK.RequestAuth
- * @returns {Object}
+ * @returns {Promise<Object>}
  */
 async function getAuthHeaders (url, auth, graphQLParams) {
   if (!auth) {
@@ -419,7 +425,7 @@ async function getAuthHeaders (url, auth, graphQLParams) {
   }
 
   return new Promise((resolve, reject) => {
-    authorizeRequest(new collectionSDK.Request({ url: new collectionSDK.Url(url), auth, body: JSON.stringify(graphQLParams) }), (err, result) => {
+    authorizeRequest(new collectionSDK.Request({ url: url, auth, method: 'POST', body: JSON.stringify(graphQLParams) }), (err, result) => {
       if (err) {
         reject(err);
       } else {
@@ -569,13 +575,14 @@ function getWSHandshakeContext (ws) {
  * Load certificate files from paths
  */
 async function loadCertificate (certificate, { cwd }) {
-  const [crt, key, pfx] = await Promise.all([
-    certificate.crt ? loadFile(certificate.crt, cwd) : undefined,
-    certificate.key ? loadFile(certificate.key, cwd) : undefined,
-    certificate.pfx ? loadFile(certificate.pfx, cwd) : undefined,
+  const [ca, cert, key, pfx] = await Promise.all([
+    certificate.ca ? loadFile(certificate.ca, { cwd }) : undefined,
+    certificate.cert ? loadFile(certificate.cert, { cwd }) : undefined,
+    certificate.key ? loadFile(certificate.key, { cwd }) : undefined,
+    certificate.pfx ? loadFile(certificate.pfx, { cwd }) : undefined,
   ]);
 
-  return { crt, key, pfx, passphrase: certificate.passphrase };
+  return { ca, cert, key, pfx, passphrase: certificate.passphrase };
 }
 
 /**
@@ -588,6 +595,18 @@ async function loadFile (path, { cwd }) {
   return await readFile(path);
 }
 
+/**
+ * Load proxy url for given url
+ */
+async function loadSystemProxy (url) {
+  return new Promise((resolve, reject) => {
+    getSystemProxy(url, (error, proxyConfig) => {
+      if (error) return reject(error);
+      resolve(proxyConfig?.getProxyUrl());
+    });
+  });
+}
+
 /**
  * This emits an event to the given channel, using the format expected by Scribe.
  */
