@@ -1,10 +1,13 @@
 // ###### WARNING: DO NOT REQUIRE NON-ISOMORPHIC LIBRARIES HERE
 let _ = require('lodash'),
+  { isString } = require('../../utils/utilities'),
   async = require('async'),
 
   EventProcessor = require('./RuntimeEventProcessor'),
   sendAnalyticsEvent = require('../../services/sendAnalytics'),
 
+  { AbortError } = require('./VaultIntegrationService'),
+
   // ###### WARNING: DO NOT REQUIRE NON-ISOMORPHIC LIBRARIES HERE
 
   // Character detection to node encoding map
@@ -30,6 +33,7 @@ let _ = require('lodash'),
   session,
   WebSocket,
   PostmanFs,
+  packageResolver,
   CookieJar,
   dryRunRequest,
   SocketIOClient,
@@ -43,8 +47,106 @@ let _ = require('lodash'),
 
   activeRuns = {},
 
+  // Object to store the abort controllers for requests in progress
+  activeRunPreparations = {},
+
   openWSConnections = {};
 
+  /**
+ * Resolves vault variables and updates environment object IN-PLACE
+ * All the secretVaulues will be returned as an Array
+ * @param {import('./VaultIntegrationService').VaultManager} vault
+ * @param {object} secrets
+ * @param {AbortSignal} abortSignal
+ * @returns {Promise<boolean>} -- if the request was aborted by user
+ */
+async function _resolveVaultVariables (vault, secrets, abortSignal) {
+  if (isBrowser) {
+    return false;
+  }
+
+  // If vault is not initialized just treat the vault integration variables as
+  // unresolved
+  if (!vault) {
+    secrets
+      .filter((secret) => secret.provider && secret.provider.id)
+      .forEach((secret) => {
+        if (_.isPlainObject(secret)) {
+          secret.value = '{{' + secret.key + '}}';
+        }
+      }
+    );
+
+    return false;
+  }
+
+
+  // Get all the secretIds to resolve
+
+  let vaultIntegrationSecrets = [];
+  let unresolvedVaultIntegrationSecrets = new Set();
+
+    if (!(Array.isArray(secrets) && secrets.length)) {
+      return false;
+    }
+
+    for (const secret of secrets) {
+      let isFromExternalProvider = secret.provider && secret.provider.id;
+      if (isFromExternalProvider) {
+        if (vault.isSupportedSecret(secret.value, secret.provider.id)) {
+          vaultIntegrationSecrets.push(secret);
+        } else {
+          unresolvedVaultIntegrationSecrets.add(secret);
+        }
+      }
+  }
+
+  // Resolve secrets
+  let resolutionResult;
+
+  try {
+    resolutionResult = await vault.resolveSecrets(vaultIntegrationSecrets.map((x) => ({
+      secretId: x.value,
+      ...(x?.metadata?.role ? { assumedRole: x.metadata.role } : {}),
+      ...(x?.metadata?.version ? { version: x.metadata.version } : {}),
+      ...(x?.metadata?.secretType ? { secretType: x.metadata.secretType } : {}),
+      ...(x?.metadata?.secretKey ? { secretKey: x.metadata.secretKey } : {}),
+      vaultId: x.provider.id
+    })), abortSignal);
+
+  } catch (e) {
+    if (e instanceof AbortError) {
+        return true;
+    }
+    throw e;
+  }
+
+  // Replace vault integration secret values with resolved value
+  for (const [idx, [secretId, result]] of Object.entries(resolutionResult)) {
+    if (result.success) {
+      vaultIntegrationSecrets[idx].value = result.value;
+    }
+    else {
+      unresolvedVaultIntegrationSecrets.add(vaultIntegrationSecrets[idx]);
+
+      if (result.error == null) continue;
+      pm.logger.warn('RuntimeExecutionService~startRun.resolveVaultVariables', {
+        secretId: secretId,
+        error: result.error.message
+      });
+    }
+  }
+  for (const unresolvedSecret of unresolvedVaultIntegrationSecrets) {
+    unresolvedSecret.value = '{{' + unresolvedSecret.key + '}}';
+  }
+
+  return false;
+}
+
+const { EventSanitizer } = require('../utils/eventSanitizer'),
+SECRET_TYPE = 'secret',
+VAULT_SCOPE = 'vault';
+
 /**
  * Helper function to get the file extension given a mime-type
  * @param {String} mimeType
@@ -309,6 +411,15 @@ function disposeRun (executionId, emit = _.noop) {
  * @param {Function} emit - emitter to call to denote the execution has terminated/disposed
  */
 function stopRun (executionId, emit) {
+  // if an abort controller exists for this request
+  // use it to stop all ongoing tasks
+  if (activeRunPreparations[executionId]) {
+    activeRunPreparations[executionId].abort();
+    delete activeRunPreparations[executionId];
+    emit({ id: executionId, event: '__dispose__' });
+    return;
+  }
+
   if (!executionId || !activeRuns[executionId]) {
     return;
   }
@@ -361,15 +472,11 @@ function resumeRun (executionId) {
  * @param {Object} variables
  * @param {Object} options
  */
-function startRun (info, collection, variables, options = {}, cookieStorageRemoteClient, emit) {
-  const sdkCollection = new postmanCollectionSdk.Collection(collection),
+async function startRun (info, collection, variables, options = {}, cookieStorageRemoteClient, vaultClient, emit) {
+  const sdkCollection = new postmanCollectionSdk.Collection(collection);
 
-    // Create an event processor instance and handling runtime events
-    eventProccessor = new EventProcessor(info.schema, (event, data, refs) => {
-      emit({ id: info.id, event, data, refs });
-    });
-
-  let cookieJar;
+  let cookieJar,
+    eventProcessor;
 
   // We have the cookieStorageRemoteClient
   if (cookieStorageRemoteClient) {
@@ -405,12 +512,78 @@ function startRun (info, collection, variables, options = {}, cookieStorageRemot
     _.set(options, 'fileResolver', new PostmanFs(workingDir || defaultWorkingDir, insecureFileRead, fileWhitelist));
   }
 
+  if (options.script && options.script.isPackageResolverEnabled) {
+    const packageResolverInstance = new packageResolver(app.getPath('temp'), collection?.info?._postman_id);
+
+    _.set(options, 'script.packageResolver', packageResolverInstance.resolvePackages);
+  }
+
+
   // sanitize
   sanitizeRunOptions(options);
 
+  // Prepare run
+  // resolve vault secrets from 3rd party vaults if any
+  // if there are no secrets from 3rd party vaults, go about the regular flow
+  if (variables.secrets?.some((secret) => secret.provider?.id)) {
+    let abort = new AbortController();
+
+    // store the abort controller for this request. This is required during
+    // stopRun to stop all ongoing tasks
+    activeRunPreparations[info.id] = abort;
+
+    let isAborted = await _resolveVaultVariables(vaultClient, variables.secrets, abort.signal);
+    if (isAborted) {
+      emit({ id: info.id, event: 'abort' });
+      return;
+    }
+
+    delete activeRunPreparations[info.id];
+  }
+
+  // Sanitize runtime events for vault variables to redact their values
+  const sensitiveWords = [];
+  let secrets = {
+    id: VAULT_SCOPE,
+    prefix: 'vault:',
+    _allowScriptAccess: _.get(options, 'script.allowVaultInScript', false),
+    values: (variables?.secrets || []).map((s) => {
+      return {
+        enabled: s.enabled,
+        type: s.type,
+        key: s.key,
+        value: s.value,
+        _domains: s.domains,
+        shouldMaskInConsole: s.shouldMaskInConsole
+      };
+    })
+  };
+
+  secrets && secrets.values && secrets.values.forEach((secret) => {
+    if (secret.type == SECRET_TYPE && isString(secret.value) && secret.shouldMaskInConsole) {
+      sensitiveWords.push(secret.value);
+    }
+  });
+
+  const eventSanitizer = new EventSanitizer(sensitiveWords);
+
+  // Create an event processor instance and handling runtime events
+  eventProcessor = new EventProcessor(info.schema, (event, data, refs) => {
+    try {
+      // Sanitize all the outgoing events against vault variables i.e. redact vault variable value
+      data = eventSanitizer.sanitizeRuntimeEvent(event, data);
+      emit({ id: info.id, event, data, refs });
+    }
+    catch (e) {
+      pm.logger.error('RuntimeExecutionService~eventSanitizer.sanitizeRuntimeEvent', e);
+    }
+  });
+
   // add variables
   variables.environment && (options.environment = new postmanCollectionSdk.VariableScope(variables.environment));
   variables.globals && (options.globals = new postmanCollectionSdk.VariableScope(variables.globals));
+  variables.secrets && (options.vaultSecrets = new postmanCollectionSdk.VariableScope(secrets));
+  variables.localVariables && (options.localVariables = variables.localVariables);
 
   // Heap memory used before start of the run.
   const startMemory = process?.memoryUsage().heapUsed;
@@ -423,26 +596,26 @@ function startRun (info, collection, variables, options = {}, cookieStorageRemot
       return;
     }
 
-    trackRun(info.id, run, eventProccessor);
+    trackRun(info.id, run, eventProcessor);
 
     // Intercept beforeRequest and response and done events
-    eventProccessor.intercept('beforeRequest', (_, __, ___, ____, aborter) => {
+    eventProcessor.intercept('beforeRequest', (_, __, ___, ____, aborter) => {
       addAborter(info.id, aborter);
     });
 
-    eventProccessor.intercept('response', (err, _, response) => {
+    eventProcessor.intercept('response', (err, _, response) => {
       removeAborter(info.id);
 
       // If the response is present, and download is set then
       if (!err && response && info.download) {
         saveStreamToFile(response.contentInfo(), response.stream, (err, success) => {
           // If there is an error success then emit the download event
-          (err || success) && eventProccessor.call('download', [err]);
+          (err || success) && eventProcessor.call('download', [err]);
         });
       }
     });
 
-    eventProccessor.intercept('done', () => {
+    eventProcessor.intercept('done', () => {
 
       // Sending the heap memory usage right before the end of the run.
       sendAnalyticsEvent('collectionrun', 'heap_memory_usage', (process.memoryUsage().heapUsed - startMemory));
@@ -460,7 +633,7 @@ function startRun (info, collection, variables, options = {}, cookieStorageRemot
 
     // Note: .handlers should be called after all interceptors have been attached
     // else events not part of schema will be ignored
-    run.start(eventProccessor.handlers());
+    run.start(eventProcessor.handlers());
   });
 }
 
@@ -469,8 +642,8 @@ function startRun (info, collection, variables, options = {}, cookieStorageRemot
  * LivePreview: Calling runtime's `dryRunRequest` to get previewed request
  *
  * @param {Request} request
- * @param {Object} info
  * @param {Object} options
+ * @param {any} cookieStorageRemoteClient
  * @param {Function} cb
  */
 function previewRequest (request, options = {}, cookieStorageRemoteClient, cb) {
@@ -521,8 +694,8 @@ function createTemporaryFile (name, content, cb) {
 
 /**
  * Read file from filesystem
- * @param {String} id
  * @param {String} path
+ * @param {Function} cb
  */
 function readFile (path, cb) {
   fs.readFile(path, (err, content) => {
@@ -541,9 +714,9 @@ function readFile (path, cb) {
 
 /**
  * Check if the given path has the required file-system access
- * @param {String} id
  * @param {String} path
- * @param {Boolean} writable
+ * @param {Boolean} writeable
+ * @param {Function} cb
  */
 function accessFile (path, writeable, cb) {
   // If no path is given then there is nothing to check
@@ -588,11 +761,14 @@ function clearAllElectronCookies (partitionId, cb) {
  * @param {Object} config
  * @param {String} config.url Server URL
  * @param {Array} config.protocols sub-protocols
- * @param {String} config.options options
- *
+ * @param {Object} config.options options
+ * @param {Object} config.certificates TLS client certificates
+ * @param {String} config.workingDir Working directory to fetch the certificates from
+ * @param {Function} cb
  */
-function wsConnect (connectionId, { url, headers, protocols = [], options = {} }, cb) {
-  const ws = new WebSocket(url, protocols, _.assign(options, { headers }));
+function wsConnect (connectionId, { url, headers, protocols = [], options = {}, certificates = {}, workingDir }, cb) {
+  const cwd = workingDir || defaultWorkingDir;
+  const ws = new WebSocket(url, protocols, _.assign(options, { headers, certificates, cwd }));
 
   openWSConnections[connectionId] = ws;
 
@@ -691,11 +867,14 @@ function wsBulkDisconnect (connectionIds = []) {
  * @param {Object} config
  * @param {String} config.url Server URL
  * @param {Array} config.protocols sub-protocols
- * @param {String} config.options options
- *
+ * @param {Object} config.options options
+ * @param {Object} config.certificates TLS client certificates
+ * @param {String} config.workingDir Working directory to fetch the certificates from
+ * @param {Function} cb
  */
-function socketIOConnect (connectionId, { url, headers, listeners = [], options = {} }, cb) {
-  const socket = new SocketIOClient(url, listeners, _.assign(options, { headers }));
+function socketIOConnect (connectionId, { url, headers, listeners = [], options = {}, certificates = {}, workingDir }, cb) {
+  const cwd = workingDir || defaultWorkingDir;
+  const socket = new SocketIOClient(url, listeners, _.assign(options, { headers, certificates, cwd }));
 
   openWSConnections[connectionId] = socket;
 
@@ -788,6 +967,111 @@ function socketIODisconnect (connectionId) {
   delete openWSConnections[connectionId];
 }
 
+/**
+ * Saves a cloud file locally.
+ *
+ * @param {string} path - The path to save the file to.
+ * @param {string|Buffer} stream - The file contents to save.
+ * @param {function} cb - The callback function to execute after saving the file.
+ * @returns {void}
+ */
+function saveCloudFileLocally (path, stream, cb) {
+
+  const directoryPath = path.substring(0, path.lastIndexOf('/'));
+
+  if (!fs.existsSync(directoryPath)) {
+    fs.mkdirSync(directoryPath, { recursive: true });
+  }
+
+  fs.writeFile(path, stream, (err) => {
+    return cb(err, true);
+  });
+}
+
+/**
+ * Saves a script file locally.
+ *
+ * @param {string} filePath - The path to save the file to.
+ * @param {string|Buffer} content - The file contents to save.
+ * @param {function} cb - The callback function to execute after saving the file.
+ * @returns {void}
+ */
+function saveScriptFileLocally (filePath, content, cb) {
+  filePath = path.resolve(app.getPath('temp'), filePath);
+
+  const directoryPath = path.dirname(filePath);
+
+  if (!fs.existsSync(directoryPath)) {
+    fs.mkdirSync(directoryPath, { recursive: true });
+  }
+
+  fs.writeFile(filePath, content, (err) => {
+    return cb(err, true);
+  });
+}
+
+/**
+ * Deletes file from local machine
+ *
+ * @param {String} path - The path of file to be deleted
+ * @param {Function} cb - The callback function to execute after successful file delete
+ * @returns {void}
+ */
+function deleteSavedCloudFile (path, cb) {
+  const directoryPath = path.substring(0, path.lastIndexOf('/'));
+
+  if (!fs.existsSync(directoryPath)) {
+    return cb(null, false);
+  }
+
+  fs.unlink(path, (err) => {
+    if (err) {
+      return cb(err, false);
+    }
+    return cb(null, true);
+  });
+}
+
+/**
+ * Deletes the cloud files directory from local
+ * @param {String} path - The path of directory which is to be deleted
+ * @param {Function} cb - The callback function to execute after successful directory delete
+ * @returns {void}
+ */
+function deleteCloudFileDir (path, cb) {
+  if (!fs.existsSync(path)) {
+    return cb(null, true);
+  }
+
+  fs.rmdirSync(path, { recursive: true, force: true }, (err) => {
+    if (err) {
+      return cb(err, false);
+    }
+    return cb(err, true);
+  });
+}
+
+/**
+ * Deletes a directory and its contents.
+ * @param {string} dirPath - The path of the directory to be deleted.
+ * @param {function} cb - The callback function to be called after the directory is deleted.
+ * @returns {void}
+ */
+function deleteScriptFileDir (dirPath, cb) {
+  const directoryPath = path.join(app.getPath('temp'), dirPath);
+
+  if (!fs.existsSync(directoryPath)) {
+    return cb(null, true);
+  }
+
+  fs.rmdirSync(directoryPath, { recursive: true, force: true }, (err) => {
+    if (err) {
+      return cb(err, false);
+    }
+    return cb(err, true);
+  });
+}
+
 // #endregion
 
 // We have two places that this is going to be used main process in electron and the we agent, hence we expose two API's here
@@ -797,6 +1081,9 @@ module.exports = function () {
   const postmanRuntime = require('postman-runtime');
   const GRPCClient = require('./GRPCClient');
   const GraphQLClient = require('./GraphQLClient');
+  const MQTTClient = require('./MQTTClient');
+  const FileSystem = require('./FileSystem');
+  const RuntimeClientUtilities = require('./RuntimeClientUtilities');
 
   fs = require('fs');
   os = require('os');
@@ -816,6 +1103,7 @@ module.exports = function () {
   getSystemProxy = require('../../utils/getSystemProxy');
 
   PostmanFs = require('../utils/postmanFs'); // This is within the common folder
+  packageResolver = require('../utils/packageResolver');
 
   collectionRunner = new postmanRuntime.Runner();
   dryRunRequest = postmanRuntime.Requester && postmanRuntime.Requester.dryRun;
@@ -849,6 +1137,9 @@ module.exports = function () {
     // Multi-protocol clients
     grpc: new GRPCClient(defaultWorkingDir),
     graphql: new GraphQLClient(defaultWorkingDir),
+    mqtt: new MQTTClient(defaultWorkingDir),
+    fileSystem: new FileSystem(defaultWorkingDir),
+    runtimeUtils: new RuntimeClientUtilities(defaultWorkingDir),
 
     // Only native
     isInWorkingDir,
@@ -857,7 +1148,12 @@ module.exports = function () {
     accessFile,
     defaultWorkingDir,
     saveStreamToFile,
-    clearAllElectronCookies
+    clearAllElectronCookies,
+    saveCloudFileLocally,
+    deleteSavedCloudFile,
+    deleteCloudFileDir,
+    deleteScriptFileDir,
+    saveScriptFileLocally
   };
 
   /* native-ignore:end */ // IMPORTANT: Do not remove this comment. Used by webpack to ignore this section
@@ -865,7 +1161,6 @@ module.exports = function () {
 
 
 module.exports.Browser = async function () {
-  // eslint-disable-next-line
   const postmanRuntime = await import(/* webpackChunkName: "postman-runtime" */ 'postman-runtime/dist');
 
   chardet = require('chardet');
@@ -889,3 +1184,7 @@ module.exports.Browser = async function () {
     saveStreamToFile
   };
 };
+
+module.exports.Test = {
+  _resolveVaultVariables,
+};
