@@ -2,8 +2,8 @@
 const DEFAULT_REQUESTER_BOUNDS = { x: null, y: null, width: 1280, height: 800 };
 const DEFAULT_RUNNER_BOUNDS = { x: null, y: null, width: 1280, height: 800 };
 const DEFAULT_CONSOLE_BOUNDS = { x: null, y: null, width: 900, height: 600 };
-const MAC_TRAFFIC_LIGHTS_POSITIONS_DEFAULT = { x: 12, y: 17 };
-const MAC_TRAFFIC_LIGHTS_Y_POSITION_FOR_ZOOM = {
+const MAC_TRAFFIC_LIGHT_POSITION_DEFAULT = { x: 12, y: 17 };
+const MAC_TRAFFIC_LIGHT_Y_POSITION_FOR_ZOOM = {
   0.25: 0,
   0.33: 1,
   0.5: 5,
@@ -23,6 +23,7 @@ const MAC_TRAFFIC_LIGHTS_Y_POSITION_FOR_ZOOM = {
   5.0: 113
 };
 const MIN_ALLOWED_WINDOW_WIDTH = 450;
+const MIN_ALLOWED_REQUESTER_WIDTH = 640;
 const MIN_ALLOWED_WINDOW_HEIGHT = 224;
 
 module.exports = {
@@ -30,7 +31,8 @@ module.exports = {
   DEFAULT_CONSOLE_BOUNDS,
   DEFAULT_RUNNER_BOUNDS,
   MIN_ALLOWED_WINDOW_WIDTH,
+  MIN_ALLOWED_REQUESTER_WIDTH,
   MIN_ALLOWED_WINDOW_HEIGHT,
-  MAC_TRAFFIC_LIGHTS_POSITIONS_DEFAULT,
-  MAC_TRAFFIC_LIGHTS_Y_POSITION_FOR_ZOOM
+  MAC_TRAFFIC_LIGHT_POSITION_DEFAULT,
+  MAC_TRAFFIC_LIGHT_Y_POSITION_FOR_ZOOM
 };
