@@ -1,5 +1,7 @@
+const { getConfig } = require('../services/AppConfigService');
+
 exports.HTML_MAP = {
-  requester: 'requester.html',
+  requester: getConfig('__WP_MODULE_FEDERATION__') ? 'workspace' : 'requester.html',
   scratchpad: 'scratchpad.html', // Requires a scratchpad build
   console: 'console.html',
   offline: 'desktop-offline.html',
@@ -31,7 +33,16 @@ exports.IPC_EVENT_NAMES = {
   GET_LAUNCH_PARAMS: 'getLaunchParams',
   GET_OFFLINE_DATA: 'getOfflineData',
   GET_PLATFORM_LAUNCH_PERF_METRICS: 'getPlatformLaunchPerfMetrics',
-  CHECK_USERS_PARTITION: 'checkUsersPartition'
+  CHECK_USERS_PARTITION: 'checkUsersPartition',
+  GET_USER_PARTITION: 'getUserPartition',
+  GET_REMOTE_INIT_PATH: 'getRemoteInitPath',
+  NAVIGATE_TO_REMOTE_URL: 'navigateToRemoteURL',
+  NAVIGATE_TO_SCRATCHPAD: 'navigateToScratchpad',
+  OPEN_EXTERNAL_URL: 'openExternalURL',
+  SCRATCHPAD_THEME_ACTION: 'scratchpadThemeAction',
+  SET_HAS_WEB_PLATFORM_NAVIGATED_TO_MONOLITH: 'setHasWebPlatformNavigatedToMonolith',
+  OPEN_NATIVE_MENU: 'setNativeElectronMenuOpen',
+  SHOW_ITEM_IN_FOLDER: 'showItemInFolder'
 };
 
 exports.EVENT_BUS_EV_NAMES = {
