@@ -1,3 +1,19 @@
+const perftraceEnvVar = process.env.PERFTRACE;
+if (typeof perftraceEnvVar === 'string' &&
+  ['true', '1'].includes(perftraceEnvVar.toLowerCase())) {
+  const { TraceEvents, trackRequires } = require('perftrace');
+
+  const traceEvents = new TraceEvents();
+
+  process.on('exit', () => {
+    const events = traceEvents.getEvents();
+    traceEvents.destroy();
+    require('fs').writeFileSync('perftrace.json', JSON.stringify(events));
+  });
+
+  trackRequires(true);
+}
+
 // IMPORTANT: This file needs to be loaded as early as possible when the app starts
 require('./preload');
 
@@ -7,79 +23,69 @@ const { AppLaunchPerfService } = require('./services/AppLaunchPerfService'),
 AppLaunchPerfService.init();
 ElectronCrashReporter.init();
 
-var electron = require('electron'),
-    fs = require('fs'),
-    semver = require('semver'),
-    app = electron.app,
-    BrowserWindow = electron.BrowserWindow,
-    Menu = electron.Menu,
-    dialog = electron.dialog,
-    circularJSON = require('circular-json'),
-    _ = require('lodash').noConflict(),
-    async = require('async'),
-    menuManager = require('./services/menuManager').menuManager,
-    windowManager = require('./services/windowManager').windowManager,
-    authHandler = require('./services/AuthHandler'),
-    authHandlerAdapter = require('./services/adapters/AuthHandlerAdapter'),
-    appSettings = require('./services/appSettings').appSettings,
-    { getOrCreateInstallationId } = require('./services/id'),
-    setupOAuth2WindowManager = require('./services/OAuth2WindowManager').initialize,
-    setupRefreshTokenManager = require('./services/AppRefreshTokenManager').initialize,
-    os = require('os'),
-    initializeEventBus = require('./common/initializeEventBus'),
-    { isAppUpdateEnabled } = require('./services/AutoUpdaterService'),
-    gpu = require('./services/gpu'),
-    updaterHandler = require('./services/UpdaterHandler'),
-    devtoolsInstaller = require('./services/devtoolsInstaller'),
-    enterpriseUtils = require('./services/enterpriseUtil'),
-    ProtocolHandler = require('./services/ProtocolHandler'),
-    ipcClient = require('./services/ipcClient').ipcClient,
-    initializeLogger = require('./services/Logger').init,
-    initializePluginsHostHandler = require('./services/PluginsHostHandler').init,
-    cleanupPluginsHostHandler = require('./services/PluginsHostHandler').cleanup,
-    CrashReporter = require('./services/CrashReporter'),
-    { getValue } = require('./utils/processArg'),
-    { createDefaultDir } = require('./services/workingDirManager'),
-    initializeAppBootListener = require('./common/services/AppBootListener'),
-    interceptorBridgeInstaller = require('./services/interceptorBridgeInstaller').installer,
-    ipcClientInitialized = false,
-    cloudProxyManager = require('./services/CloudProxyManager'),
-    leaderSelection = require('./services/LeaderSelection'),
-    proxyCertificateService = require('./services/proxyCertificateService'),
-    ProxyMainManager = require('./services/ProxyMainManager'),
-    VersionDependencyService = require('./services/VersionDependencyService'),
-    architectureService = require('./services/ArchitectureService'),
-    WindowController = require('./common/controllers/WindowController'),
-    proxyMainManager = new ProxyMainManager(),
-    FSProcessManager = require('./services/integrations/file-system/ProcessManager'),
-    git = require('./services/integrations/git'),
-    path = require('path'),
-    { performShellDataMigration, isShellMigrationDone } = require('./services/shellDataMigration'),
-    authDataInterface = require('./services/authDataInterface'),
-    {
-      MAC_TRAFFIC_LIGHTS_POSITIONS_DEFAULT,
-      MAC_TRAFFIC_LIGHTS_Y_POSITION_FOR_ZOOM
-    } = require('./constants/WindowConstants'),
-    { runTimeoutPromise } = require('./services/automation-testing/utils/promise'),
-
-    // Initialising file system interfaces and attaching handlers to the interface events via a newly spawned process
-    FSManager = new FSProcessManager();
-
+// Node.JS dependencies
+const fs = require('fs');
+const os = require('os');
+const path = require('path');
 const { performance } = require('perf_hooks');
-const { PERF_MEASURES, PERF_MARKS } = require('./constants/PerformanceAnalyticsConstants');
-const { EVENT_BUS_EV_NAMES } = require('./constants');
+
+// Third party dependencies
+const { app, BrowserWindow, Menu, dialog, contentTracing, shell, nativeTheme } = require('electron');
+const semver = require('semver');
+const circularJSON = require('circular-json');
+const _ = require('lodash').noConflict();
+const async = require('async');
+const jsonStorage = require('electron-json-storage');
+
+// Internal dependencies
+const menuManager = require('./services/menuManager').menuManager;
+const windowManager = require('./services/windowManager').windowManager;
+const authHandler = require('./services/AuthHandler');
+const appSettings = require('./services/appSettings').appSettings;
+const proxyUtils = require('./utils/proxy/utils').utils;
+const { getOrCreateInstallationId } = require('./services/id');
+const setupOAuth2WindowManager = require('./services/OAuth2WindowManager').initialize;
+const setupRefreshTokenManager = require('./services/AppRefreshTokenManager').initialize;
+const initializeEventBus = require('./common/initializeEventBus');
+const { isAppUpdateEnabled } = require('./services/AutoUpdaterService');
+const gpu = require('./services/gpu');
+const updaterHandler = require('./services/UpdaterHandler');
+const enterpriseUtils = require('./services/enterpriseUtil');
+const ProtocolHandler = require('./services/ProtocolHandler');
+const interceptorConnection = require('./services/InterceptorConnection').interceptorConnection;
+const initializeLogger = require('./services/Logger').init;
+const CrashReporter = require('./services/CrashReporter');
+const { getValue } = require('./utils/processArg');
+const { createDefaultDir } = require('./services/workingDirManager');
+const initializeAppBootListener = require('./common/services/AppBootListener');
+const interceptorBridgeInstaller = require('./services/interceptorBridgeInstaller').installer;
+const cloudProxyManager = require('./services/CloudProxyManager');
+const leaderSelection = require('./services/LeaderSelection');
+const proxyCertificateService = require('./services/proxyCertificateService').proxyCertificateService;
+const ProxyMainManager = require('./services/ProxyMainManager');
+const VersionDependencyService = require('./services/VersionDependencyService');
+const architectureService = require('./services/ArchitectureService');
+const WindowController = require('./common/controllers/WindowController');
+const { performShellDataMigration, isShellMigrationDone } = require('./services/shellDataMigration');
+const { runTimeoutPromise } = require('./services/automation-testing/utils/promise');
+const i18nService = require('./services/i18n/i18nService');
 const authPartitionService = require('./services/authPartitionService');
+
+// Constants
+const {
+  MAC_TRAFFIC_LIGHT_POSITION_DEFAULT,
+  MAC_TRAFFIC_LIGHT_Y_POSITION_FOR_ZOOM
+} = require('./constants/WindowConstants');
+const { PERF_MEASURES, PERF_MARKS } = require('./constants/PerformanceAnalyticsConstants');
 const { IPC_EVENT_NAMES } = require('./constants');
-
-const MOVE_DIALOG_MESSAGE = 'Move to Applications Folder?',
-      MOVE_DIALOG_ACTION_BUTTON = 'Move to Applications Folder',
-      MOVE_DIALOG_CANCEL_BUTTON = 'Do Not Move',
-      MOVE_DIALOG_CHECKBOX_MESSAGE = 'Do not remind me again',
-      MOVE_DIALOG_DETAILS = 'I can move myself to the Applications folder if you\'d like. This will ensure that future updates will be installed correctly.',
-
-      SUPPORT_LINK = 'https://go.pstmn.io/troubleshoot-could-not-open',
-      COULD_NOT_OPEN_DIALOG_TITLE = 'Could not open Postman',
-      COULD_NOT_OPEN_DIALOG_MESSAGE = 'Please try restarting the app. If the issue persists, please refer to ' + SUPPORT_LINK;
+const MOVE_DIALOG_MESSAGE = 'Move to Applications Folder?';
+const MOVE_DIALOG_ACTION_BUTTON = 'Move to Applications Folder';
+const MOVE_DIALOG_CANCEL_BUTTON = 'Do Not Move';
+const MOVE_DIALOG_CHECKBOX_MESSAGE = 'Do not remind me again';
+const MOVE_DIALOG_DETAILS = 'I can move myself to the Applications folder if you\'d like. This will ensure that future updates will be installed correctly.';
+const SUPPORT_LINK = 'https://go.pstmn.io/troubleshoot-could-not-open';
+const COULD_NOT_OPEN_DIALOG_TITLE = 'Could not open Postman';
+const COULD_NOT_OPEN_DIALOG_MESSAGE = 'Please try restarting the app. If the issue persists, please refer to ' + SUPPORT_LINK;
 
 // We upgraded Electron to v7 that comes with node v12 in Postman v7.25.1. The default minimum
 // supported version of TLS in node v10 was v1 which was changed to v1.2 in node v12
@@ -88,14 +94,22 @@ const MOVE_DIALOG_MESSAGE = 'Move to Applications Folder?',
 // As a fix, we are overriding this default minimum version of TLS back to v1
 require('tls').DEFAULT_MIN_VERSION = 'TLSv1';
 
-process.env.LOCAL_GIT_DIRECTORY = path.resolve(process.resourcesPath, 'data', 'git');
+// https://postmanlabs.atlassian.net/browse/INC-735
+// We uppgraded Electron 28 that comes with node v18 in Postman v11.6.0. Node v17 introduced a
+// change where the default result order for DNS lookups was passed as-is to the underlying call.
+// This caused an issue where the user's with servers bound to the IPV4 addresses 0.0.0.0 were not
+// able to connect due to the DNS lookup returning an IPV6 address.
+// As a fix, we are overriding the default result order for DNS lookups to ipv4first
+require('dns').setDefaultResultOrder('ipv4first');
 
 performance.mark(PERF_MARKS.MAIN_PROCESS_IMPORTS_COMPLETE);
 performance.measure(PERF_MEASURES.MAIN_PROCESS_IMPORTS_TIME);
 
+const proxyMainManager = new ProxyMainManager();
 const ipcMain = pm.sdk.IPC;
-
 let executionProcess;
+let ipcClientInitialized = false;
+let webSocketServerInitialized = false;
 
 try {
   // set the path for the directory storing app's configuration files
@@ -107,17 +121,19 @@ catch (e) {
 
 app.sessionId = process.pid; // set the current process id as sessionId
 
-let gotTheLock = app.requestSingleInstanceLock();
 
-// Quit if this is another instance of app trying to open
-if (!gotTheLock) {
-  pm.logger.info('Could not get the lock, quitting');
-  app.quit();
+(function () {
+  let gotTheLock = app.requestSingleInstanceLock();
+
+  // Quit if this is another instance of app trying to open
+  if (!gotTheLock) {
+    pm.logger.info('Could not get the lock, quitting');
+    app.quit();
+
+    return;
+  }
+})();
 
-  // NOTE: once we turn on minification, this will throw an error
-  // We can use an IIFE or an else clause
-  return;
-}
 
 // Initializing ProtocolHandler as early as possible to handle Run-In-Postman even when the app is closed.
 // Recommended to do it on will-finish-launching event in electron docs - https://electronjs.org/docs/api/app#event-will-finish-launching
@@ -141,15 +157,30 @@ if (gpu.shouldDisableGPU()) {
  *
  * They are executed after the first window is opened.
  */
-function deferredInitSteps (cb) {
-  let RuntimeIPCAgent = require('./runtime/agents/IPCAgent'),
-      RuntimeExecutionService = require('./common/services/RuntimeExecutionService');
+function deferredInitSteps () {
+  let initialising = false;
 
-  RuntimeIPCAgent.start(RuntimeExecutionService(), (e) => {
-    if (e) {
-      pm.logger.warn('main.js: Could not initialise runtime module', e);
+  pm.eventBus.channel('runtime-ipc-lifecycle').subscribe((event) => {
+    if (event.namespace === 'ipcAgent' && event.name === 'queryStatus') {
+      if (!initialising) {
+        initialising = true;
+
+        const RuntimeIPCAgent = require('./runtime/agents/IPCAgent'),
+          RuntimeExecutionService = require('./common/services/RuntimeExecutionService');
+
+        RuntimeIPCAgent.start(RuntimeExecutionService(), (e) => {
+          if (e) {
+            pm.logger.warn('main.js: Could not initialise runtime module', e);
+          }
+
+          // if a window was opened before the agent could initialize itself
+          // we have to broadcast an event to all of them that agent is initialized.
+          pm.eventBus.channel('runtime-ipc-lifecycle').publish({ name: 'statusBroadcast', namespace: 'ipcAgent' });
+        });
+      } else {
+        pm.eventBus.channel('runtime-ipc-lifecycle').publish({ name: 'statusBroadcast', namespace: 'ipcAgent' });
+      }
     }
-    cb(e);
   });
 }
 
@@ -158,7 +189,7 @@ function deferredInitSteps (cb) {
  * sequence of the app and only needs to be triggered after the user has expressed an intent to trigger some sort of
  * collection execution -- a performance test, a collection run, or something else.
  *
- * @returns {boolean} True if the process is available for use, false otherwise.
+ * @returns {Promise<boolean>} True if the process is available for use, false otherwise.
  */
 async function bootExecutionProcess ({ type, options }) {
   try {
@@ -203,6 +234,64 @@ async.series([
     cb();
   },
 
+  (cb) => {
+    try {
+      // Perform cleanup of redundant storage file + lock files. We do this early in the boot lifecycle because
+      // we only want to cleanup stale lockfiles here. So, we do this here to avoid deleting any actual
+      // lock files
+      // Presence of stale lock files created by electron-json-storage causes an issue where multiple write
+      // operations lead to a deadlock and the CPU usage of the app spikes
+      // Refer to - https://github.com/postmanlabs/postman-app-support/issues/7294#issuecomment-1669001260
+
+      pm.logger.info('Main: Starting cleanup of stale files');
+
+      // List of lock files to remove
+      const LOCK_FILES_LIST = [
+        'requester.json.lock',
+        'settings.json.lock',
+        'userPartitionData.json.lock',
+        'newAccountRegionSettings.json.lock'
+      ],
+        DATA_PATH = jsonStorage.getDataPath();
+
+      // Remove the requester storage file (This is a redundant legacy file which is not used anymore)
+      const requesterRemovePromise = new Promise((resolve, reject) => {
+        jsonStorage.remove('requester', (err) => {
+          if (err) {
+            pm.logger.info('Main: Cleanup of requester.json file failed. This is non-blocking for app boot', err);
+            reject(err);
+          }
+
+          resolve();
+        });
+      });
+
+      // Remove each of the lockfiles
+      const lockFilesCleanupPromises = LOCK_FILES_LIST.map((lockFile) => {
+        const filePath = path.resolve(DATA_PATH, lockFile);
+
+        return fs.promises.rm(filePath)
+          .catch((err) => {
+            // If the lock file does not exist, it's expected, so it's fine. Otherwise
+            // we log the error
+            if (err.code !== 'ENOENT') {
+              pm.logger.info('Main: Lock file could not be deleted - ', filePath, err);
+            }
+          });
+      });
+
+      // We use Promise.allSettled here as even if the cleanup fails, we don't want to block the app boot
+      Promise.allSettled([requesterRemovePromise, ...lockFilesCleanupPromises])
+        .then((results) => {
+          pm.logger.info('Main: Cleanup of stale files completed');
+          cb();
+        });
+    }
+    catch (e) {
+      cb();
+    }
+  },
+
   /**
    * Try creating the default working directory if running for the first time
    */
@@ -246,7 +335,11 @@ async.series([
 
   (cb) => {
     // Initialize the devtools installer
-    devtoolsInstaller.init(cb);
+    if (process.env.PM_BUILD_ENV === 'development' || false) {
+      require('./services/devtoolsInstaller').init(cb);
+    } else {
+      cb(null);
+    }
   },
 
   /**
@@ -303,9 +396,6 @@ async.series([
       });
     }
   },
-
-  initializePluginsHostHandler
-
 ], (err) => {
 
   if (err) {
@@ -393,11 +483,33 @@ async.series([
               value: false
             }
           });
+
+          // sending internal message to renderer to update cert trust status as the user will have to re trust the certificates
+          windowManager.sendInternalMessage({
+            event: 'certsRegenerated',
+            'object': {
+              certTrusted: false
+            }
+          });
         })
         .catch((e) => {
           pm.logger.error('Main~IPC-MessageToElectron - Error regenerating certificates', e);
         });
       }
+      else if (arg.event === 'installCertificates') {
+        proxyUtils.installAndTrustCertificate();
+      }
+      else if (arg.event === 'isCertAddedToKeyChain') {
+        proxyUtils.isCertificateAddedToKeyChain()
+          .then((isAdded) => {
+            windowManager.sendInternalMessage({
+              event: 'certAdditionStatus',
+              'object': {
+                value: isAdded
+              }
+            });
+          });
+      }
       else if (arg.event === 'startProxy') {
         var port = 8080;
         if (arg.data && arg.data.port) {
@@ -412,10 +524,10 @@ async.series([
           proxyMainManager.startProxyProcess({
             env: {
               port,
+              useImprovedFTUX: arg.data.useImprovedFTUX,
               STORE_LOC: app.getPath('userData'),
               ...process.env
             },
-            inspect: false
           });
 
           // TODO: handle proxy close on user log-out
@@ -497,56 +609,53 @@ async.series([
         interceptorBridgeInstaller.resetInstallation();
       }
       else if (arg.event === 'forwardInterceptorRequest') {
-        ipcClient.sendEncryptedMessageToInterceptor(arg.message);
+        interceptorConnection.sendEncryptedMessageToInterceptor(arg.message);
       }
       else if (arg.event === 'initializeInterceptorBridge') {
         // creating IPC bridge with native server
         if (!ipcClientInitialized) {
-          ipcClient.initialize();
+          interceptorConnection.initializeIpcClient();
           ipcClientInitialized = true;
         }
-        else {
-          windowManager.sendInternalMessage({
-            'event': 'refreshInterceptorBridgeConnectionStatus',
-            'object': {
-              connectedToInterceptorBridge: true
-            }
-          });
+      }
+      else if (arg.event === 'initializeWebSocketServer') {
+        if (!webSocketServerInitialized) {
+          interceptorConnection.initializeWebSocketServer(arg.message);
+          webSocketServerInitialized = true;
         }
       }
       else if (arg.event === 'disconnectInterceptorBridge') {
-        ipcClient.disconnect();
+        interceptorConnection.disconnect();
+      }
+      else if (arg.event === 'changeActiveConnection') {
+        interceptorConnection.changeActiveConnection(arg.message);
+      }
+      else if (arg.event === 'sendActiveSessionStatus') {
+        interceptorConnection.sendActiveSessionStatus(arg.message);
       }
-
       else if (arg.event === 'setEncryptionKeyForInterceptor') {
         // setting the encryption key for App ~ Interceptor communication
-        ipcClient.setEncryptionKeyForInterceptor(arg.message.encryptionKeyForInterceptor);
+        interceptorConnection.setEncryptionKeyForInterceptor(arg.message.encryptionKeyForInterceptor);
 
         if (ipcClientInitialized) {
           // checking for the same key at interceptor
-          ipcClient.startKeyValidationFlow();
+          interceptorConnection.startKeyValidationFlow();
         }
       }
       else if (arg.event === 'getEncryptionKeyForInterceptor') {
-        ipcClient.sendEncryptionKeyToRenderer();
+        interceptorConnection.sendEncryptionKeyToRenderer();
       }
       else if (arg.event === 'getSyncDomainListForInterceptor') {
-        ipcClient.sendSyncDomainListToRenderer();
+        interceptorConnection.sendSyncDomainListToRenderer();
       }
       else if (arg.event === 'fetchCustomEncryptionKey') {
-        ipcClient.sendCustomEncryptionKeyToRenderer();
+        interceptorConnection.sendCustomEncryptionKeyToRenderer();
       }
       else if (arg.event === 'removeEncryption') {
-        ipcClient.removeCustomEncryptionKeyFromInterceptor();
+        interceptorConnection.removeCustomEncryptionKeyFromInterceptor();
       }
     });
 
-    // Local git operations
-    ipcMain.handle('performGitOperation', async (event, arg) => {
-      const result = await git.gitFactory(arg.data);
-      return result;
-    });
-
     ipcMain.handle('getSaveTarget', function (event, arg) {
       return new Promise((resolve, reject) => {
         showSaveDialog(event.sender, arg, (retPath) => {
@@ -569,6 +678,14 @@ async.series([
       });
     });
 
+    ipcMain.handle('set-native-theme-source', (event, theme) => {
+      nativeTheme.themeSource = theme;
+    });
+
+    ipcMain.handle('check-native-theme-source', (event, theme) => {
+      return nativeTheme.themeSource !== theme;
+    });
+
     ipcMain.subscribe('sendToAllWindows', function (event, arg) {
       try {
         let parsedArg = circularJSON.parse(arg);
@@ -597,7 +714,12 @@ async.series([
     });
 
     ipcMain.subscribe('closeRequesterWindow', function (event, arg) {
-      windowManager.closedHandler(event, arg);
+      // Since the window manager expects a window, not a WebContents instance, this call is broken.
+      // The change I've made here ensures that the code is exactly as broken as it was before.
+      // For all I know, this event handler can't even be reached without reaching for DevTools.
+      // However, removing it is a larger change touching many other components.
+      // The same goes for 'closeWindow', too, and potentially other handlers around these two.
+      windowManager.notifyOfWindowClose(event.sender, event);
     });
 
     ipcMain.subscribe('closeWindow', (event, arg) => {
@@ -624,12 +746,21 @@ async.series([
         }
 
         const currentWindow = BrowserWindow.getFocusedWindow();
-        const newTrafficLightPositions = {
-          x: MAC_TRAFFIC_LIGHTS_POSITIONS_DEFAULT.x * arg,
-          y: MAC_TRAFFIC_LIGHTS_Y_POSITION_FOR_ZOOM[arg] || MAC_TRAFFIC_LIGHTS_POSITIONS_DEFAULT.y * arg
+        const newTrafficLightPosition = {
+          x: MAC_TRAFFIC_LIGHT_POSITION_DEFAULT.x * arg,
+          y: MAC_TRAFFIC_LIGHT_Y_POSITION_FOR_ZOOM[arg] || MAC_TRAFFIC_LIGHT_POSITION_DEFAULT.y * arg
         };
 
-        currentWindow && currentWindow.setTrafficLightPosition(newTrafficLightPositions);
+        // We're currently checking if we need to use the old or the new API.
+        // Once we're on Electron v25 or newer, we can remove the check and use the new API.
+        // Also renaming constants and variables doesn't make sense, however.
+        // A "traffic light" is more descriptive than "window buttons".
+        // Electron refers to it elsewhere, too, and so do some Postman's UI controls.
+        if (currentWindow?.setWindowButtonPosition) {
+          currentWindow.setWindowButtonPosition(newTrafficLightPosition);
+        } else {
+          currentWindow?.setTrafficLightPosition(newTrafficLightPosition);
+        }
       });
     }
 
@@ -683,6 +814,37 @@ async.series([
     ipcMain.handle('bootExecutionProcess', async (event, arg) => {
       return await bootExecutionProcess(arg);
     });
+
+    let hiddenWindow = null;
+
+    ipcMain.handle('handle-export-as-PDF', async (event, arg) => {
+      return new Promise((resolve, reject) => {
+        hiddenWindow = new BrowserWindow({ show: false });
+
+        hiddenWindow.loadURL(`data:text/html;charset=utf-8,${encodeURIComponent(arg.data)}`);
+
+        hiddenWindow.webContents.session.on('will-download', (event, item, webContents) => {
+          // Set the save path, making Electron not to prompt a save dialog.
+          item.setSavePath(arg.name);
+
+          item.once('done', (event, state) => {
+            if (hiddenWindow && !hiddenWindow.isDestroyed()) {
+              hiddenWindow.close();
+              hiddenWindow = null;
+            }
+
+            if (state !== 'completed') {
+              reject(new Error(`Download failed: ${state}`));
+            }
+
+            resolve();
+          });
+        });
+      });
+    });
+
+    // To invoke an item in a folder via electron shell using the full path
+    ipcMain.handle(IPC_EVENT_NAMES.SHOW_ITEM_IN_FOLDER, (event, fullPath) => shell.showItemInFolder(fullPath));
   }
 
   process.on('uncaughtException', function (e) {
@@ -690,7 +852,6 @@ async.series([
   });
 
   /**
-   *
    * @param {*} e
    */
   function handleUncaughtError (e) {
@@ -700,9 +861,7 @@ async.series([
   }
 
   /**
-   *
    * @param {*} action
-   * @param {*} url
    */
   function runPostmanShortcut (action) {
     if (action == 'newWindow') {
@@ -717,6 +876,10 @@ async.series([
 
   // Quit when all windows are closed.
   app.on('window-all-closed', function () {
+     // Kill proxy process when all windows are closed.
+     // As renderer TCP reader will not be listening to proxied traffic anymore
+     proxyMainManager.killProxyProcess();
+
     // If all the windows are closed, we quit the application only on linux and windows platform.
     // We don't want to quit the application when user choose to switch b/w multiple accounts
     if (process.platform != 'darwin' && windowManager.isFirstRequesterBooted) {
@@ -725,12 +888,9 @@ async.series([
     }
   });
 
-
   app.on('web-contents-created', (event, contents) => {
     if (contents.getType() === 'webview') {
-      contents.on('new-window', (event) => {
-        event.preventDefault();
-      });
+      contents.setWindowOpenHandler(() => ({ action: 'deny' }));
     }
   });
 
@@ -748,9 +908,6 @@ async.series([
     // Kill proxy before quitting
     proxyMainManager.killProxyProcess();
 
-    // terminate the plugin process
-    cleanupPluginsHostHandler();
-
     if (os.type() !== 'Linux') {
       app.updaterInstance = null;
     }
@@ -772,7 +929,6 @@ async.series([
   });
 
   /**
-   *
    * @param {*} window
    * @param {*} fileName
    */
@@ -929,7 +1085,7 @@ async.series([
 
   /**
    * Updates the version on the app about panel
-   * @param {String} version
+   * @param {String} productVersion
    * @param {String} UIVersion
    */
   function updateVersionOnAppAboutPanel (productVersion, UIVersion) {
@@ -944,99 +1100,115 @@ async.series([
    * This will be called when app is ready
    */
   function onAppReady () {
+    if (false) {
+      pm.logger.info('Tracing enabled');
+      contentTracing.startRecording({
+        included_categories: ['*']
+      })
+        .then(async () => {
+          pm.logger.info('Tracing started');
+          await new Promise((resolve) => setTimeout(resolve, 10000));
+          const tracePath = await contentTracing.stopRecording();
+          pm.logger.info('Tracing data recorded to ' + tracePath);
+        });
+    }
     performance.mark(PERF_MARKS.MAIN_PROCESS_APP_READY);
     performance.measure(PERF_MEASURES.MAIN_PROCESS_APP_READY_TIME, PERF_MARKS.MAIN_PROCESS_IMPORTS_COMPLETE);
 
-    promptMoveToApplicationsFolder(() => {
-      // Populates the installation id
-      populateInstallationId();
-      populateUpdateSettings();
-
-      let runningDeferredInitSteps = false;
-
-      let unsubscribe = eventBus.channel('main-internal-lifecycle').subscribe((event) => {
-        if (!event || event.namespace != 'main-internal-lifecycle' || event.name !== EVENT_BUS_EV_NAMES.WINDOW_LOADED) {
-          return;
+    const initI18nServiceWithRetry = async (tries = 3) => {
+      try {
+        // Initialize i18n in main process
+        await i18nService.init();
+      } catch (err) {
+        if (tries <= 0) {
+          // If there are no more tries left, throw the error
+          throw err;
         }
 
-        // flag to prevent running this step more than once
-        if (runningDeferredInitSteps) {
-          return;
-        }
-        runningDeferredInitSteps = true;
+        // If there are more tries left, retry
+        return initI18nServiceWithRetry(tries - 1);
+      }
+    };
+
+    // Initialize i18n service with retry and, if it fails, log the error and continue with post i18n init operations
+    initI18nServiceWithRetry(3).catch((err) => {
+      pm.logger.error('main~onAppReady - Failed to initialize i18n service', err);
+    }).finally(() => {
+
+      promptMoveToApplicationsFolder(() => {
+        // Populates the installation id
+        populateInstallationId();
+        populateUpdateSettings();
 
         // run deferred init steps
-        deferredInitSteps((e) => {
-          runningDeferredInitSteps = false;
+        deferredInitSteps();
 
-          // unsubscribe handler to prevent running steps for next requester window
-          unsubscribe();
+        Promise.resolve()
+          .then(() => {
+            if (process.env.SKIP_SIGNIN !== 'true') {
+              return authHandler.init();
+            }
+          })
+          .then(() => {
+            return WindowController.initialize({ eventBus });
+          })
+          .then(() => {
+            const appEvents = eventBus.channel('app-events');
+            appEvents.subscribe((event = {}) => {
+              // If it is a boot process;
+              if (_.get(event, 'name') === 'booted') {
+                let process = event.namespace,
+                    err = event.data,
+                    meta = event.meta;
 
-          if (e) {
-            pm.logger.warn('main.js: Could not complete deferred init steps', e);
-          }
-        });
-      });
+                if (process === 'requester' && meta && meta.isFirstRequester) {
+                  if (err) {
+                    pm.logger.error('Main~AppEvents - First requester window boot failed', err);
 
-      // initialize event bus
-      let appEvents = eventBus.channel('app-events');
+                    let errorMessageToShow = err.name + ': ' + err.message;
+                    dialog.showErrorBox(COULD_NOT_OPEN_DIALOG_TITLE, errorMessageToShow + '\n\n' + COULD_NOT_OPEN_DIALOG_MESSAGE);
 
-      Promise.resolve()
-        .then(() => {
-          if (process.env.SKIP_SIGNIN !== 'true') {
-            return authHandler.init(authHandlerAdapter);
-          }
-        })
-        .then(() => {
-          return WindowController.initialize({ eventBus });
-        })
-        .then(() => {
-          appEvents.subscribe((event = {}) => {
-            // If it is a boot process;
-            if (_.get(event, 'name') === 'booted') {
-              let process = event.namespace,
-                  err = event.data,
-                  meta = event.meta;
+                    return;
+                  }
 
-              if (process === 'requester' && meta && meta.isFirstRequester) {
-                if (err) {
-                  pm.logger.error('Main~AppEvents - First requester window boot failed', err);
-
-                  let errorMessageToShow = err.name + ': ' + err.message;
-                  dialog.showErrorBox(COULD_NOT_OPEN_DIALOG_TITLE, errorMessageToShow + '\n\n' + COULD_NOT_OPEN_DIALOG_MESSAGE);
-
-                  return;
+                  updateVersionOnAppAboutPanel(meta.productVersion, meta.UIVersion);
                 }
 
-                updateVersionOnAppAboutPanel(meta.productVersion, meta.UIVersion);
+                pm.logger.info(`Main~AppEvents - Received booted event for process ${process}.Version ${meta.productVersion} UI Version: ${meta.UIVersion} Desktop Platform Version: ${app.getVersion()}`);
               }
+            });
 
-              pm.logger.info(`Main~AppEvents - Received booted event for process ${process}.Version ${meta.productVersion} UI Version: ${meta.UIVersion} Desktop Platform Version: ${app.getVersion()}`);
-            }
+            windowManager.openRequesterWindows();
           });
 
-          windowManager.openRequesterWindows();
+        appSettings.set('downloadedVersion', null);
+
+        shouldHaveShortcuts((err, shortcutsDisabled) => {
+          app.shortcutsDisabled = shortcutsDisabled;
+          menuManager.createMenu(shortcutsDisabled);
         });
 
-      appSettings.set('downloadedVersion', null);
-
-
-      shouldHaveShortcuts((err, shortcutsDisabled) => {
-        app.shortcutsDisabled = shortcutsDisabled;
-        menuManager.createMenu(shortcutsDisabled);
+        if (app.dock) { // app.dock is only available on OSX
+          app.dock.setMenu(dockMenu);
+        }
       });
 
-      if (app.dock) { // app.dock is only available on OSX
-        app.dock.setMenu(dockMenu);
-      }
+      proxyCertificateService.generateRootCAForProxy(app.getPath('userData'))
+        .then((result) => {
+          if (result.certsRegenerated) {
+            // sending internal message to renderer to update cert trust status as the user will have to re trust the certificates
+            windowManager.sendInternalMessage({
+              event: 'certsRegenerated',
+              'object': {
+                certTrusted: false
+              }
+            });
+          }
+        })
+        .catch((e) => {
+          pm.logger.error('HTTPSProxy~Unable to generate certificates', e);
+        });
     });
-
-    // Generate rootCA certificates for HTTPS Proxy when app is ready
-    try {
-      proxyCertificateService.generateRootCAForProxy(app.getPath('userData'));
-    } catch (e) {
-      pm.logger.error('HTTPSProxy~Unable to generate certificates', e);
-    }
   }
 
   // This method will be called when Electron has done everything
@@ -1053,7 +1225,7 @@ async.series([
     .getOpenWindows('requester')
     .then((openRequesterWindows) => {
       // if there are open requester windows do nothing
-      if (!_.isEmpty(openRequesterWindows)) {
+      if (Array.isArray(openRequesterWindows) && openRequesterWindows.length > 0) {
         return;
       }
 
