@@ -1,15 +1,31 @@
 const _ = require('lodash');
-const os = require('os');
 const system = require('../../lib/system');
+const { SCENARIO_EXECUTION_EVENTS } = require('../constants');
 const ThreadGroup = require('./ThreadGroup');
 const { createError, CONSTANTS } = require('../../lib/communication');
 const { log } = require('../../lib/logging');
 const { killAllChildProcesses } = require('../../lib/process-management');
+const ExtendedHeap = require('./ExtendedHeap');
 
 const WORKER_MIN_ENDURANCE = 7; // This will be set when a workload is assigned to Engine
 const WORKER_MAX_ENDURANCE = 10;
 const HEAP_LIMIT_PER_PROCESS = 1024 * 1024 * 1024; // 1 GB
 
+/**
+ * This is the strategy for heaping the threads in the active pool.
+ * This will be used to heapify threads in the active pool such that the thread with
+ * the least assigned scenarios will be at the top of the heap.
+ *
+ * @param {Object} threadA - Thread object
+ * @param {Object} threadB - Thread object
+ *
+ * @returns {Number} - 1 if threadA is greater than threadB, -1 if threadA is less than threadB, 0 if threadA is equal to threadB
+ */
+const activeThreadsHeapingStrategy = (threadA, threadB) => {
+  return threadA.assigned - threadB.assigned;
+};
+
+
 /**
  * The ExecutionEngine class is responsible for distributing the workload across the worker threads to achieve
  * parallelism. It is also responsible for managing the lifecycle of the worker threads such that the worker threads
@@ -45,18 +61,27 @@ module.exports = class ExecutionEngine {
   // to be started.
   pools = {
     reserve: new Map(),
-    active: new Map(),
+
+    // ExtendedHeap is equivalent to Heap class with added ability to make constant time lookups for nodes.
+    // This class will heapify the threads based on the number of scenarios assigned to them. Which will
+    // be used to distribute the workload across the threads in a least utilization manner.
+    // The thread with the least number of assigned scenarios will be at the top of the heap and will be chosen for the
+    // next assignment in O(1) time.
+    active: new ExtendedHeap(activeThreadsHeapingStrategy),
     draining: new Map(),
   };
 
+  // This will be a map of sequence ids to the thread ids. This will be used to kill the assigned
+  // scenario on a thread. Engine will use this to figure out the container and thread for a sequence,
+  // to be able to create a signal for killing it. Using this it will determine the correct container and
+  // correct thread inside container for targeting the signal.
+  sequenceAssignmentMap = new Map();
+
   // This will be a map of thread group ids to the thread group objects. This will be used to manage the lifecycle of
   // the thread groups. We will be starting and stopping the thread groups based on the workload assigned to them in a
   // round-robin fashion.
   groups = new Map();
-
-  // This will be a map of scenario ids to the thread group ids. This will be used to assign scenarios to the thread
-  // groups.
-  activePoolSize = 0; // This will be set when startup is called
+  poolSize = 0; // This will be set when startup is called
 
   // Pointer for holding reference to any pre-constructed instance of this class to enforce a singleton of this class.
   static instance = null;
@@ -98,10 +123,25 @@ module.exports = class ExecutionEngine {
   _getNumberOfPossibleGroups = async () => {
     // Decide number of processes to spawn based on free memory of the system as we want to allocate more memory to
     // the threads. We will be spawning threads in round robin across containers.
-    // TODO: Check if the getPlatformSpecs() can fail or not. If it can fail, then we need to handle that case.
-    const { memory, logicalCores } = await system.getPlatformSpecs();
 
-    let numberOfGroups = Math.floor((memory.available) / HEAP_LIMIT_PER_PROCESS);
+    const specs = await system.getPlatformSpecs();
+    const FALLBACK_AVAILABLE_MEMORY = 0;
+    const FALLBACK_LOGICAL_CORES = 2;
+
+    if (!specs?.memory?.available || !specs?.logicalCores) {
+      log.warn('[Engine] Unable to detect system specs. Execution will fall back to minimum sensible defaults for ' +
+        `memory (${FALLBACK_AVAILABLE_MEMORY}) and logical cores (${FALLBACK_LOGICAL_CORES})`);
+    }
+
+    // Free memory is used to calculate the number of processes we'll be spawning. If this information is not available
+    // we can spawn at least one process safely.
+    const availableMemory = specs?.memory?.available || FALLBACK_AVAILABLE_MEMORY;
+
+    // Fall back to a reasonable number of logical cores if we're unable to fetch any reliable information about the
+    // host. Most hosts (at least those being used by actual Postman users) will have at least 2 logical cores.
+    const logicalCores = specs?.logicalCores || FALLBACK_LOGICAL_CORES;
+
+    let numberOfGroups = Math.floor((availableMemory) / HEAP_LIMIT_PER_PROCESS);
 
     // If the number of groups is less than 1, then we will spawn only 1 process as the container.
     if (numberOfGroups < 1) {
@@ -128,15 +168,19 @@ module.exports = class ExecutionEngine {
   setup = async () => {
     const { numberOfGroups, numberOfThreads } = await this._getNumberOfPossibleGroups();
 
+    log.info(`[Engine] Starting ${numberOfGroups} groups and ${numberOfThreads} threads`);
+
     // We will be dividing number of threads to start equally among the number of groups.
     const groups = await this.spawnGroups(numberOfGroups);
 
+    log.info('[Engine] Spawned all the groups');
+
     for (const group of groups) {
       this.groups.set(group.id, group);
     }
 
     // Total number of threads to spawn will be the number of logical CPUs available on the machine.
-    this.activePoolSize = numberOfThreads;
+    this.poolSize = numberOfThreads;
   }
 
   /**
@@ -154,6 +198,8 @@ module.exports = class ExecutionEngine {
       return;
     }
 
+    log.info(`[Engine] Spawning ${numberOfGroups - this.groups.size} more groups to adjust for available memory`);
+
     const groups = await this.spawnGroups(numberOfGroups - this.groups.size);
 
     for (const group of groups) {
@@ -213,6 +259,8 @@ module.exports = class ExecutionEngine {
 
     await threadGroup.init();
 
+    log.info(`[Engine] Started and initialized a thread-group with ID ${id}`);
+
     return threadGroup;
   }
 
@@ -244,6 +292,8 @@ module.exports = class ExecutionEngine {
       // Set the maximum endurance to 10% more than the minimum endurance
       this.maximumWorkerEndurance = Math.ceil(endurance * 1.10);
     }
+
+    log.info(`[Engine] Setting up the endurance to ${this.minimumWorkerEndurance} - ${this.maximumWorkerEndurance}`);
   }
 
   /**
@@ -274,6 +324,13 @@ module.exports = class ExecutionEngine {
       log.info(`Killing worker from ${poolName} pool, ${id}`);
       workerItem.worker.stop();
       this.pools[poolName].delete(id);
+
+      // Clear the sequenceId to workerId mapping for the killed worker
+      this.sequenceAssignmentMap.forEach((value, key) => {
+        if (value === id) {
+          this.sequenceAssignmentMap.delete(key);
+        }
+      });
     }
     else {
       log.warn(`Execution - ExecutionEngine ~ Attempted to kill worker which doesn't exist, ${id}`);
@@ -291,7 +348,7 @@ module.exports = class ExecutionEngine {
    * @returns {Number|*}
    */
   get desiredPoolSize () {
-    return os.cpus().length;
+    return this.poolSize;
   }
 
   /**
@@ -324,9 +381,7 @@ module.exports = class ExecutionEngine {
     ]);
 
     // add all the workers to the active pool and reserved pool to begin with
-    for (const worker of activeThreads) {
-      this.pools.active.set(worker.id, worker);
-    }
+    this.pools.active.addAll(activeThreads);
 
     for (const worker of reservedThreads) {
       this.pools.reserve.set(worker.id, worker);
@@ -336,7 +391,7 @@ module.exports = class ExecutionEngine {
     // to the draining pool based on the number of runs they have serviced. Drained workers will be killed.
     // For each killed thread, one is started into the reserve pool. If the active pool does not have "enough"
     // workers, some will be moved from reserve to active. Based on what?
-    log.info(`Execution - ExecutionEngine.init ~ Started ${activeThreads.length + reservedThreads.length} workers.`);
+    log.info(`Execution - ExecutionEngine.init ~ Started ${activeThreads.length} + ${reservedThreads.length} workers.`);
 
     this.initializing = false;
     this.initialized = true;
@@ -354,18 +409,20 @@ module.exports = class ExecutionEngine {
     this.options?.runtimeEventHandler?.(data);
 
     // Handle lifecycle events
-    this.handleRuntimeEvents(data?.event, data?.executionContext?.executorReferenceId);
+    this.handleRuntimeEvents(data?.event, data?.executionContext);
   }
 
   /**
    * Event handler for runtime events from the workers to manage workload among workers
    *
    * @param {Object} executorEvent - The event from the executor worker to listen for runtime events
-   * @param {String} workerId - The workerId of the worker that emitted the event
+   * @param {Object} executionContext - The execution context of the worker that emitted the event
+   * @param {String} executionContext.executorReferenceId - The ID of the worker that emitted the event
+   * @param {String} executionContext.sequenceId - The ID of the sequence that was executed
    *
    * @returns {void}
    */
-  handleRuntimeEvents = (executorEvent, workerId) => {
+  handleRuntimeEvents = (executorEvent, { executorReferenceId: workerId, sequenceId }) => {
     let workerItem, poolName;
 
     Object.keys(this.pools).forEach((pool) => {
@@ -382,15 +439,28 @@ module.exports = class ExecutionEngine {
     }
 
     switch (executorEvent) {
-      case 'done':
-      case 'error':
-        // worker might belong to any of the pools - active or draining
-        const worker = (this.pools.active.get(workerId) || this.pools.draining.get(workerId));
+      case SCENARIO_EXECUTION_EVENTS.FINISHED:
+      case SCENARIO_EXECUTION_EVENTS.ABORTED:
+        // remove sequenceId from the sequenceAssignmentMap
+        this.sequenceAssignmentMap.delete(sequenceId);
 
-        if (worker) {
-          worker.finished += 1;
+        // Break statement is not required here as we need to do the same thing for both the events.
+        // We will preserve the sequenceId to workerId mapping in case of error event so mapping
+        // will be only cleaned when finished or aborted events is issued.
+        // eslint-disable-next-line no-fallthrough
 
-          if (worker.finished >= worker.assigned && this.pools.draining.has(workerId)) {
+      case SCENARIO_EXECUTION_EVENTS.ERROR:
+        if (workerItem) {
+          workerItem.finished += 1;
+          workerItem.assigned -= 1;
+
+          if (this.pools.active.has(workerId)) {
+            // If the worker is in the active pool, then we need to heapify the active pool again to make sure that
+            // the thread with the least number of assigned scenarios is at the top of the heap.
+            this.pools.active.update(workerItem);
+          }
+
+          if (workerItem.finished >= workerItem.totalAssignments && this.pools.draining.has(workerId)) {
             this.disposeThread(workerId);
           }
         }
@@ -415,8 +485,10 @@ module.exports = class ExecutionEngine {
 
     if (worker) {
       this.pools.draining.set(workerId, worker);
-      this.pools.active.delete(workerId);
+      this.pools.active.remove(workerId);
       await this.activateReservedThread();
+
+      log.debug(`[Engine] Moved worker ${workerId} from active to draining`);
     }
     else {
       log.warn(`Execution - ExecutionEngine ~ Trying to drain worker not in active pool, ${workerId}`);
@@ -441,13 +513,17 @@ module.exports = class ExecutionEngine {
       });
     }
 
-    const [reserveId, reserveProcess] = this.pools.reserve.entries()?.next().value;
+    const [reserveId, reserveThread] = this.pools.reserve.entries()?.next().value;
 
-    this.pools.active.set(reserveId, reserveProcess);
+    this.pools.active.add(reserveThread);
     this.pools.reserve.delete(reserveId);
 
+    log.debug(`[Engine] Moved worker ${reserveId} from reserve to active`);
+
     // Start a new worker to replace the one we just removed from the reserve
     await this.startThread();
+
+    log.debug('[Engine] Spawned a replacement thread into the reserve pool');
   }
 
   /**
@@ -461,6 +537,8 @@ module.exports = class ExecutionEngine {
     if (worker) {
       this.stopThread(workerId);
       this.pools.draining.delete(workerId);
+
+      log.debug(`[Engine] Disposed a thread with ID ${workerId}`);
     }
     else {
       log.warn(`Execution - ExecutionEngine ~ Trying to dispose worker not in draining pool, ${workerId}`);
@@ -468,16 +546,12 @@ module.exports = class ExecutionEngine {
   }
 
   /**
-   * Assign a scenario to a worker-thread. Since scenarios are uniform in nature for the moment, we can assume that each
-   * scenario will take about as much time to finish as adjacent ones. This means we can start off with a simple round
-   * robin load balancing algorithm across workers.
-   *
-   * @TODO: Switch to a "least utilization" algorithm since it's probably the right balance of complexity and
-   * effectiveness.
+   * Assign a scenario to a worker-thread. It uses least utilization algorithm to assign the scenario to the worker.
    *
    * @TODO: Introduce the idea of partitioning across different bundles when we need to support more than one type of
    * execution at a time.
    *
+   * @param  {object} params
    * @param  {object} params.executionContext - An opaque object used accompanying this scenario.
    * @param  {object} params.data - Data to be passed to the scenario worker to be used in the execution.
    */
@@ -502,10 +576,9 @@ module.exports = class ExecutionEngine {
 
     // Assign this scenario to the worker. the assignment pointer is referencing, then advance the pointer to continue
     // the round-robin assignment.
-    let nextActiveThread = this.getNextActiveThread();
+    let leastAssignedThread = this.pools.active.peek();
 
-    if (!nextActiveThread) {
-      // @TODO: Panic and exit, properly surfacing the error. This should never happen.
+    if (!leastAssignedThread) {
       createError({
         message: 'ExecutionEngine ~ No active worker available for scenario assignment',
         source: 'assignScenario',
@@ -514,20 +587,98 @@ module.exports = class ExecutionEngine {
       });
     }
 
-    nextActiveThread.assigned += 1;
+    leastAssignedThread.assigned += 1;
+    leastAssignedThread.totalAssignments += 1;
 
-    const workerId = nextActiveThread.id;
+    // Replace the least assigned thread in the heap to make sure that the thread with the least number of assigned
+    // scenarios is at the top of the heap.
+    this.pools.active.update(leastAssignedThread);
 
-    nextActiveThread.worker.assignRun(args);
+    const workerId = leastAssignedThread.id;
+
+    leastAssignedThread.worker.assignRun(args);
+
+    const sequenceId = args[0].executionContext.sequenceId;
+
+    log.debug(`[Engine] Assigned scenario ${sequenceId} to worker ${workerId}`);
+
+    // Store the sequenceId to workerId mapping for killing the scenario, if needed
+    this.sequenceAssignmentMap.set(sequenceId, workerId);
 
     // Each worker is drained once it has reached its configured endurance to provide some protection against leaks
     // within individual workers. Check with >= rather than === to catch any workers which may have fallen
     // through the cracks. Using === here can lead to, at some point in the future, workers never being drained.
-    if (nextActiveThread.assigned >= nextActiveThread.endurance) {
+    if (leastAssignedThread.totalAssignments >= leastAssignedThread.endurance) {
       await this.drainThread(workerId);
     }
   };
 
+  /**
+   * Aborts a scenario execution for a given sequenceId.
+   *
+   * @param {Object} executionContext - The execution context of the scenario to kill.
+   * @param {String} executionContext.sequenceId - The sequenceId of the scenario to kill.
+   * @param {String} executionContext.executionId - The executionId of the scenario to kill.
+   *
+   * @returns {Promise<*>} - Returns a promise which will get resolved when the request is submitted to the worker.
+   */
+  abortScenario = async (executionContext) => {
+    const { sequenceId } = executionContext;
+
+    // Get the workerId to target the signal for killing the scenario
+    const workerId = this.sequenceAssignmentMap.get(sequenceId);
+
+    if (!workerId) {
+      log.warn(`Execution - ExecutionEngine ~ Trying to abort scenario that is not assigned to any worker, ${sequenceId}`);
+      return;
+    }
+
+    // The scenario could be in one of the thread of active or draining pool, so get the reference to the worker
+    // from any of the pools.
+    const workerItem = this.pools.active.get(workerId) || this.pools.draining.get(workerId);
+
+    if (!workerItem) {
+      log.warn(`Execution - ExecutionEngine ~ Trying to abort scenario that is not assigned to any worker, ${sequenceId}`);
+      return;
+    }
+
+    log.info(`Execution - ExecutionEngine ~ Killing scenario of sequenceId [${sequenceId}] from worker [${workerId}]`);
+
+    return workerItem.worker.abortRun(executionContext);
+  }
+
+  /**
+   * Kills a scenario execution for a given sequenceId.
+   *
+   * @param {Object} executionContext - The execution context of the scenario to kill.
+   * @param {String} executionContext.sequenceId - The sequenceId of the scenario to kill.
+   * @param {String} executionContext.executionId - The executionId of the scenario to kill.
+   *
+   * @returns {Promise<*>} - Returns a promise which will get resolved when the request is submitted to the worker.
+   */
+  stopScenario = async (executionContext) => {
+      const { sequenceId } = executionContext;
+
+      // Get the workerId to target the signal for killing the scenario
+      const workerId = this.sequenceAssignmentMap.get(sequenceId);
+
+      if (!workerId) {
+        log.warn(`Execution - ExecutionEngine ~ Trying to stop scenario that is not assigned to any worker, ${sequenceId}`);
+        return;
+      }
+
+      // The scenario could be in one of the thread of active or draining pool, so get the reference to the worker
+      // from any of the pools.
+      const threadGroupRef = this.pools.active.get(workerId) || this.pools.draining.get(workerId);
+
+      if (!threadGroupRef) {
+        log.warn(`Execution - ExecutionEngine ~ Trying to stop scenario that is not assigned to any worker, ${sequenceId}`);
+        return;
+      }
+
+      return threadGroupRef.worker.stopRun(executionContext);
+    }
+
   /**
    * Starts all the workers configured.
    *
@@ -538,27 +689,6 @@ module.exports = class ExecutionEngine {
     return Promise.all(Array.from({ length }, () => this.startWorkerThread()));
   };
 
-  /**
-   * Uses an Iterator to cycle through the started workers. It will recreate the Iterator when necessary since
-   * Iterator objects are not cyclic by nature.
-   */
-  getNextActiveThread = () => {
-    if (typeof this.assignmentPointer?.next !== 'function') {
-      // Initial assignment, in case assignmentPointer isn't already implementing an Iterator.
-      this.assignmentPointer = this.pools.active.values();
-    }
-
-    let next = this.assignmentPointer.next();
-
-    // Check if we're exhausted the Iterator and refresh it if so.
-    if (next.done) {
-      this.assignmentPointer = this.pools.active.values();
-      next = this.assignmentPointer.next();
-    }
-
-    return next.value;
-  };
-
   /**
    * Starts a worker thread and adds it to the active pool. If the worker manager is stopped while the worker is
    * starting, we need to kill the worker. Not doing so might cause the worker to be orphaned and continue running.
@@ -579,6 +709,8 @@ module.exports = class ExecutionEngine {
 
     const worker = await group.createThread();
 
+    log.info(`[Engine] Started a worker ${worker.id} in group ${group.id}`);
+
     if (this.stopped) {
       // If the worker manager is stopped while the worker is starting, we need to kill the worker.
       // Not doing so might cause the worker to be orphaned and continue running.
@@ -603,20 +735,24 @@ module.exports = class ExecutionEngine {
 
       // Since this is a new worker, it's not assigned to any scenarios yet.
       assigned: 0,
-      finished: 0
+      finished: 0,
+      totalAssignments: 0
     };
   }
 
   /**
    * Kills all the workers.
    * @memberof ExecutionEngine
-   * @returns {void}
+   * @returns {Promise<void>}
    */
   stopAll = async () => {
     // Set the stopped flag to true so that no new workers are started or assignments are allowed
     this.stopped = true;
 
     try {
+      // Clear the sequence to worker mapping
+      this.sequenceAssignmentMap.clear();
+
       // Ask all the groups to stop their workers
       // Promise.allSettled is used here to ensure that all the workers are killed even if one of them fails to kill
       const results = await Promise.allSettled(Array.from(this.groups.values()).map((group) => group.stop()));
@@ -635,6 +771,9 @@ module.exports = class ExecutionEngine {
           severity: CONSTANTS.SEVERITY.ERROR
         });
       }
+      else {
+        log.info('[Engine] Successfully stopped all the threads in groups');
+      }
     } catch (e) {
       log.error('ExecutionEngine.killAll ~ Error while killing all workers', e);
     } finally {
@@ -666,7 +805,7 @@ module.exports = class ExecutionEngine {
       createError({
         message: 'Error while force killing child processes',
         error: e,
-        source: 'processManager.forceKill',
+        source: 'Engine.forceKill',
         subsystem: 'ScenarioManager',
         severity: CONSTANTS.SEVERITY.CRITICAL
       });
@@ -723,6 +862,7 @@ module.exports = class ExecutionEngine {
     const newThreadGroup = await this.createThreadGroup(this.getUnusedGroupId());
 
     let thread;
+
     const threadCount = threadGroup.threads.size;
 
     // replace each thread of the process that dies with a new thread
@@ -735,6 +875,8 @@ module.exports = class ExecutionEngine {
     }));
 
     this.groups.set(newThreadGroup.id, newThreadGroup);
+
+    log.info(`Spawned a new thread group with ID ${newThreadGroup.id} and its threads`);
   }
 
   unInit = () => {
