@@ -7,15 +7,23 @@
  * @TODO: What abstraction do we use to wrap around a group of scenarios? Do we even need to introduce the idea of
  * a scenario within this class or do we use something more traditional like "transaction"?
  */
+const _ = require('lodash');
 const system = require('../lib/system');
 const { log } = require('../lib/logging');
 const loadProfiles = require('./load-profile');
 const Sequence = require('./Sequence');
-const { WORKLOAD_STATES } = require('../ScenarioManager/constants');
+const {
+  WORKLOAD_STATES,
+  DATAFILE_DISTRIBUTION_TYPES,
+  DATAFILE_RANDOM_DISTRIBUTION
+} = require('../ScenarioManager/constants');
 const { broadcastOnIPC } = require('../sub-process-functions');
 const { deserializeObject } = require('../utils/string');
 const ResourceUtilizationQueue = require('./resource-utilization-queue');
 const { getUtilizationConfigDelta } = require('./resource-utilization-tuner');
+const { createError, CONSTANTS } = require('../lib/communication');
+const SequenceTracker = require('./SequenceTracker');
+const LoadProfile = require('./load-profile/LoadProfile');
 
 const defaults = {
   // Load profiles are continuous functions but it's not necessary to continually evaluate them since fractional virtual
@@ -46,6 +54,11 @@ const defaults = {
   // will be forcibly terminated.
   gracePeriodMs: 5000,
 
+  // The time we allow for scenarios to gracefully finish after a scenario has been requested to do so.
+  // The grace period allows some flexibility to allow this, but also enforces an absolute maximum duration
+  // beyond which scenarios and sequences will be forcibly terminated.
+  sequenceGracePeriod: 2000,
+
   // Number of resource utilization measurements to depend upon to make decisions about the configs.
   // This number is not set in stone, we can change based on requirement in future.
   resourceUtilObservationCount: 5,
@@ -74,6 +87,8 @@ const parseContext = (context) => {
 };
 
 module.exports = class WorkloadManager {
+  dataFileConfig = null;
+
   /**
    * Creates a new instance of the WorkloadManager which can be used for the duration of the execution. Multiple
    * executions can occur in parallel, and each execution goes through a specific lifecycle:
@@ -87,23 +102,25 @@ module.exports = class WorkloadManager {
    *
    * @TODO: Does the WorkloadManager need to be aware of what's happening within each scenario?
    *
+   * @param  {Object} params
    * @param  {Array} params.sequence - A set of request or Item (as defined by postman-collection) IDs representing
    *                                   the series of requests to be executed.
+   * @param  {Object} params.context
    * @param  {[String]} params.context.collection - The collection ID. This is an array for now to support multiple
    *                                                collections in the future.
    * @param  {String} params.context.environment - The environment ID.
-   * @param  {String} params.options.loadProfile - A pre-defined load profile, as identified by the load profile
-   *                                               definition.
-   * @param  {Number} params.options.duration - The maximum duration of this execution.
-   * @param  {Number} params.options.maxVUs - The maximum number of virtual users to be simulated during this execution.
+   * @param  {Object} params.options
+   * @param  {String} params.options.loadProfile - POJO representation of selected load profile which is the return
+   *                                               value of LoadProfile base class `toObject` method
+   *                                               (xMax is duration of this execution in minutes, yMax is max VUs)
    */
   constructor ({ sequence, context: rawContext, options }) {
     const context = parseContext(rawContext);
 
     // Validate the inputs before we do absolutely anything
     // @TODO: Consolidate and expand this validation logic elsewhere and invoke instead of having it inline.
-    if (!options.loadProfile || !loadProfiles[options.loadProfile]) {
-      throw new Error(`Valid load profile selection is required, but got ${options.loadProfile}.`);
+    if (!options.loadProfile || !options.loadProfile.id || !loadProfiles[options.loadProfile.id]) {
+      throw new Error(`Valid load profile selection is required, but got ${options.loadProfile.id}.`);
     }
 
     // A unique ID, to be used *within this session*, to reference this instance of the WorkloadManager.
@@ -112,29 +129,36 @@ module.exports = class WorkloadManager {
 
     this.config = Object.assign({}, defaults, {
       ...options,
-      loadProfile: loadProfiles[options.loadProfile],
-      duration: options.duration,
-      maxVUs: options.maxVUs,
-      rampDurationMs: options.rampDurationMs,
+      loadProfile: options.loadProfile,
+      durationMs: options.loadProfile.xMax,
       collection: context.collection,
       requestSelection: context.requestSelection,
       environment: context.environment || null,
       globals: context.globals || null,
+      isTransient: options.isTransient,
       runtimeOptions: options.runtimeOptions || {}, // Refer TODO above about merging with default config
     });
 
+    if (context.dataFileConfig) {
+      this.setDataFileConfig(context.dataFileConfig);
+    }
+
     // @TODO: Consolidate these under something that can represent the current state of this WorkloadManager instance.
     this.eventLoopTimeout = null;
     this.startTime = null;
     this.executionId = this.workloadId;
     this.executionContext = { executionContextId: this.workloadId };
-    this.currentUsersCount = null;
     this.runOngoing = false;
-    this.deadzoneThresholdMs = options.duration - this.config.deadZoneMs;
-    this.gracefulDurationMs = options.duration + this.config.gracePeriodMs;
+    this.deadzoneThresholdMs = this.config.durationMs - this.config.deadZoneMs;
+    this.gracefulDurationMs = this.config.durationMs + this.config.gracePeriodMs;
     this.sequences = new Set();
     this.sequenceIdMap = new Map();
     this.currentSleepDuration = defaults.sleepTimeMs;
+    this.lastCreatedSequenceIndex = null;
+
+    // Using an array and not a set here to maintain the order in which the sequences were stopped
+    // and to use the same order when re-using the indexes available
+    this.usableSequenceIndexes = [];
 
     // Print platform specs again such that if the logs are rotated on the machine, last run start event can convey
     // the platform specs.
@@ -143,6 +167,8 @@ module.exports = class WorkloadManager {
     });
 
     this.resourceUtilizationQueue = new ResourceUtilizationQueue(defaults.resourceUtilObservationCount);
+
+    this.trackers = [];
   }
 
   /**
@@ -157,30 +183,41 @@ module.exports = class WorkloadManager {
     // maintained along with IPC channels to communicate with the workers.
 
     try {
-      pm.logger.info('WorkloadManager~Init initializing');
+      pm.logger.info(`[WorkloadManager][${this.workloadId}]~Init initializing`);
 
-      await global.SUBSYSTEM_REGISTER.private.scenarioManager.init(this.config.maxVUs);
+      await system.invoke('scenarioManager', {
+        method: 'init',
+        args: [this.config.loadProfile.yMax]
+      });
 
       this._setState(WORKLOAD_STATES.READY);
+
+      log.debug(`[WorkloadManager][${this.workloadId}] Workload is ready to start run`);
     }
     catch (error) {
-      pm.logger.error('WorkloadManager~Error initializing: ', error);
+      pm.logger.error(`[WorkloadManager][${this.workloadId}]~Error initializing: `, error);
 
       // cleanup and enter error state, so that it can be retried
+      // TODO: Change this to use system.invoke instead of using globals directly.
       await global.SUBSYSTEM_REGISTER.private.scenarioManager.kill();
 
-      this._setState(WORKLOAD_STATES.ERROR);
+      this._setState(WORKLOAD_STATES.ERROR, error);
     }
   }
 
-  _setState = (state) => {
-    if (Object.values(WORKLOAD_STATES).includes(state)) {
+  _setState = (state, error = null) => {
+    if (Object.values(WORKLOAD_STATES).includes(state) && state !== this.state) {
       this.state = state;
+      const workloadError = error ? {
+        name: error.name,
+        message: error.message
+      } : null;
 
       // TODO: Refactor the data payload once we have uniform structure for all the events.
       broadcastOnIPC('workload-state-change', {
         id: this.workloadId,
-        state: this.state
+        state: this.state,
+        error: workloadError
       });
     }
   }
@@ -197,6 +234,23 @@ module.exports = class WorkloadManager {
     await this.stop(WORKLOAD_STATES.TERMINATED);
   }
 
+  /**
+   * Function to get an integer ID to be given to a Sequence object. It tries to fetch
+   * an ID from a pool called `usableSequenceIndexes`. If the pool is empty, it returns
+   * an incremented value that tracks the highest value of sequence-index used so far.
+   *
+   * @returns {number} - The sequence index to be used as the ID which creating sequences
+   */
+  getSequenceIndex = () => {
+    if (this.usableSequenceIndexes.length > 0) {
+      return this.usableSequenceIndexes.shift();
+    }
+
+    else {
+      return this.lastCreatedSequenceIndex++;
+    }
+  }
+
   /**
    * The Workload event loop runs continuously once an execution has started and ensures that:
    *
@@ -220,10 +274,22 @@ module.exports = class WorkloadManager {
    */
   async workloadEventLoop () {
     const tickTime = new Date();
-    const elapsedMs = tickTime - this.startTime;
 
-    // Record the system utilization and store them in queue.
-    this.resourceUtilizationQueue.enqueue(await system.getUsageStats());
+    // If elapsed time is 0 set it to 1ms.
+    // Since the incrementDelay is 2s, if we get the elapsed time as 0 then no sequences will be started
+    // in the first tick. This will lead to a delay of 2s in starting the scenarios. This is not desirable
+    // and hence we are setting the elapsed time to 1ms.
+    const elapsedMs = (tickTime - this.startTime) <= 0 ? 1 : (tickTime - this.startTime);
+
+    // Record the system utilization and store them in queue without blocking the event loop
+    system.getUsageStats()
+      .then(this.resourceUtilizationQueue.enqueue)
+      .catch((err) => log.error(
+          `WorkloadManager[${this.workloadId}]~Error while getting system utilization`,
+          err.toString(),
+          err.stack
+        )
+      );
 
     // If resource utilization measurement queue is full, update sleep duration
     if (this.resourceUtilizationQueue.isFull()) {
@@ -234,44 +300,59 @@ module.exports = class WorkloadManager {
     // Workload duration.
     if (elapsedMs < this.deadzoneThresholdMs) {
       const actualSequenceCount = this.sequences.size;
-      const desiredSequenceCount = this.config.loadProfile.evaluate({
-        totalDurationMs: this.config.duration,
-        maxLoad: this.config.maxVUs,
-        rampDurationMs: this.config.rampDurationMs,
+      const desiredSequenceCount = LoadProfile.evaluate({
+        loadProfile: this.config.loadProfile,
         elapsedMs
       });
 
+      let totalTracked = 0;
+
+      for (const tracker of this.trackers) {
+        totalTracked += tracker.target;
+      }
+
       // @TODO: Account for load profiles where the load can decrease as well. This implementation only accounts
       // for loads which remain constant or increase over time. In case a profile gets added without resolving this
       // todo, having Math.max here will ensure that the rest of the code doesn't fail catastrophically.
-      const sequenceDelta = Math.max(desiredSequenceCount - actualSequenceCount, 0);
+      const sequenceDelta = desiredSequenceCount - actualSequenceCount + totalTracked;
 
       if (sequenceDelta > 0) {
         for (let member = 1; member <= sequenceDelta; member++) {
           const sequence = new Sequence({
+            workloadSequenceIndex: this.getSequenceIndex(),
             sequence: this.config.collection,
             environment: this.config.environment,
             globals: this.config.globals,
             requestSelection: this.config.requestSelection,
             runOptions: this.config.runtimeOptions,
             executionContext: this.executionContext,
-            getSleepDuration: () => { return this.getSleepDuration(); } // We need closure here to maintain the scope.
+            getSleepDuration: () => { return this.getSleepDuration(); }, // We need closure here to maintain the scope.
+            sequenceFinishCallback: this.sequenceFinishCallback.bind(this),
+            sequenceOptions: {
+              sequenceGracePeriod: this.config.sequenceGracePeriod
+            }
           });
 
           this.sequences.add(sequence);
           this.sequenceIdMap.set(sequence.id, sequence);
 
           await sequence.start();
-
-          await system.invoke('metricsProcessor', {
-            method: 'processSetVUCount',
-            args: [{
-              executionContext: this.executionContext,
-              state: 'running'
-            }]
-          });
         }
       }
+      else if (sequenceDelta < 0) {
+        const tracker = new SequenceTracker(-sequenceDelta, this.config.gracePeriodMs, this.abortSequences.bind(this));
+
+        this.trackers.push(tracker);
+      }
+
+      // Set the current VU count in the metric store
+      await system.invoke('metricsProcessor', {
+        method: 'processSetVUCount',
+        args: [{
+          executionContext: this.executionContext,
+          currentVuCount: desiredSequenceCount,
+        }]
+      });
     }
 
     if (elapsedMs < this.gracefulDurationMs) {
@@ -283,10 +364,13 @@ module.exports = class WorkloadManager {
       if (elapsedMs >= this.deadzoneThresholdMs) {
         // all the scenarios have completed
         if (this.sequenceIdMap.size === 0) {
+          log.debug(`WorkloadManager[${this.workloadId}]~All scenarios have completed, stopping execution`);
           this.stop(WORKLOAD_STATES.FINISHED);
         }
         else {
           // this is where we need to let the sequences know that they should not start any new scenario executions
+          log.debug(`WorkloadManager[${this.workloadId}]~Duration has exceeded deadzone threshold, stopping new scenarios`);
+
           for (let [sequenceId, sequence] of this.sequenceIdMap) {
             sequence.stopNewScenarios();
           }
@@ -307,6 +391,7 @@ module.exports = class WorkloadManager {
       // all the scenarios have completed
       // As of now, we are killing all the sequences. In future, we need to kill only the sequences that are belonging
       // to a particular execution context
+      log.debug(`WorkloadManager[${this.workloadId}]~Duration has exceeded grace period, stopping execution`);
       this.stop(WORKLOAD_STATES.FINISHED);
 
       // Since no longer relevant, we will kill the queue
@@ -314,6 +399,51 @@ module.exports = class WorkloadManager {
     }
   }
 
+  sequenceFinishCallback (sequenceId, state) {
+    // It is possible that the first tracker exhausted its grace period, force-stopped
+    // the remaining Sequences and set its target to 0. So, we'll need to find a tracker
+    // whose target is a positive number and keep clearing the ones that are done.
+    while (this.trackers.length > 0 && this.trackers[0].target === 0) {
+      this.trackers.shift();
+    }
+
+    if (state !== 'aborted') {
+      if (this.trackers.length > 0) {
+        this.trackers[0].countDown();
+
+        if (this.trackers[0].target === 0) {
+          this.trackers.shift();
+        }
+      } else {
+        return false;
+      }
+    }
+
+    // Preserve the index of the sequence that will be removed, to reuse if a new sequence is added later
+    this.usableSequenceIndexes.push(this.sequenceIdMap.get(sequenceId).workloadSequenceIndex);
+
+    // Delete the records of the sequence from the set and the map
+    this.sequences.delete(this.sequenceIdMap.get(sequenceId));
+    this.sequenceIdMap.delete(sequenceId);
+
+    return true;
+  }
+
+  /**
+   * Aborts the given number of sequences by aborting the runs in whichever point
+   * of execution they are in.
+   *
+   * @param {number} count - The number of sequences to stop
+   */
+  async abortSequences (count = 0) {
+    if (this.sequences.size > 0) {
+      const sequences = Array.from(this.sequences.values()).filter((seq) => !seq.sequenceCompleted).slice(0, count);
+      for (let sequence of sequences) {
+        await sequence.abortScenario();
+      }
+    }
+  }
+
   /**
    * This returns the sleep delay based on current config, and resource utilization.
    *
@@ -366,7 +496,7 @@ module.exports = class WorkloadManager {
 
     if (this.sequenceIdMap.size > 0) {
       for (let [_, sequence] of this.sequenceIdMap) {
-        sequence.killScenario();
+        sequence.abortScenario();
       }
     }
 
@@ -378,6 +508,8 @@ module.exports = class WorkloadManager {
     // @TODO: The WorkloadManager shouldn't be directly aware of scenarios, but it should signal the Sequence instead.
     await system.invoke('scenarioManager', { method: 'stop' });
 
+    log.debug('[WorkloadManager] ScenarioManager was stopped');
+
     system.releasePowerSaver();
 
     this._setState(finalState);
@@ -387,14 +519,23 @@ module.exports = class WorkloadManager {
    * This will be called when the Workload is being destroyed. This is the place to do any cleanup work that needs to be
    * done before the Workload is removed from memory.
    */
-  destroy () {
+  destroy = async () => {
+    // clear off all the sequence trackers after the workload is destroyed
+    this.trackers = [];
+
     // TODO: Add clean up for listeners of workload manager and all the sequences
+    // Reset token manager if that was associated with this workload instance
+    await system.invoke('tokenManager', {
+      method: 'reset',
+      args: [this.workloadId]
+    });
   }
 
   /**
    * Recovers a workload from any error states
    */
   recover = () => {
+    log.info(`[WorkloadManager][${this.workloadId}] Workload is being recovered`);
     return this.init();
   }
 
@@ -403,9 +544,8 @@ module.exports = class WorkloadManager {
       ...this.config,
       requestSelection: this.config.requestSelection,
       loadProfile: this.config.loadProfile.id, // this contains the name of the profile: constant, ramp etc.
-      duration: this.config.duration,
-      maxVUs: this.config.maxVUs,
-      rampDurationMs: this.config.rampDurationMs,
+      duration: this.config.durationMs,
+      maxVUs: this.config.loadProfile.yMax,
       runtimeOptions: this.config.runtimeOptions,
       startTime: this.startTime,
       finishTime: this.finishTime
@@ -417,24 +557,24 @@ module.exports = class WorkloadManager {
    * loop. This method then only needs to do some data wrangling and trigger the initial set of scenario executions.
    */
   async start () {
-    pm.logger.info('WorkloadManager ~ About to start the workload run');
+    pm.logger.info(`WorkloadManager[${this.workloadId}] ~ About to start the workload run`);
     this._setState(WORKLOAD_STATES.STARTING);
 
-    pm.logger.info('WorkloadManager ~ Preventing power saver from kicking in');
+    pm.logger.info(`WorkloadManager[${this.workloadId}] ~ Preventing power saver from kicking in`);
     system.preventPowerSaver(this.gracefulDurationMs);
 
     // @TODO: Add validation steps before starting the Workload.
     this.startTime = new Date();
     this.runOngoing = true;
 
-    pm.logger.info('WorkloadManager ~ Listening to events of sequences completing');
+    pm.logger.info(`WorkloadManager[${this.workloadId}] ~ Listening to events of sequences completing`);
 
     // Kick off the Workload's event loop. This will adjust the state of the workload every interval to keep it close
     // to the load profile.
-    pm.logger.info('WorkloadManager ~ Entering the event-loop of the workload');
+    pm.logger.info(`WorkloadManager[${this.workloadId}] ~ Entering the event-loop of the workload`);
     this.workloadEventLoop();
 
-    pm.logger.info('WorkloadManager ~ Setting the WLM state to RUNNING');
+    pm.logger.info(`WorkloadManager[${this.workloadId}] ~ Setting the WLM state to RUNNING`);
     this._setState(WORKLOAD_STATES.RUNNING);
   }
 
@@ -452,13 +592,128 @@ module.exports = class WorkloadManager {
       await this.sequenceIdMap.get(sequenceId).handleExecutorCallback(executorResponse);
     }
     else {
-      pm.logger.info('Sequence completed, not handling the executor response', sequenceId);
+      pm.logger.info(`WorkloadManager[${this.workloadId}] Sequence completed, not handling the executor response`, sequenceId);
     }
   }
 
   async processSequenceEvents (sequenceEvent) {
     if (sequenceEvent.state === 'completed') {
+      log.debug(`WorkloadManager[${this.workloadId}] ~ Sequence ${sequenceEvent.sequenceId} completed`);
+
+      // Preserve the index of the sequence that will be removed, to reuse if a new sequence is added later
+      this.usableSequenceIndexes.push(this.sequenceIdMap.get(sequenceEvent.sequenceId).workloadSequenceIndex);
+
+      this.sequences.delete(this.sequenceIdMap.get(sequenceEvent.sequenceId));
       this.sequenceIdMap.delete(sequenceEvent.sequenceId);
     }
   }
+
+  /**
+   * Sets the data file configuration for the workload.
+   *
+   * @param {object} dataFileConfig - Data file configuration.
+   *
+   * @throws {Error} - If data file config is already set or if data file config is not valid.
+   *
+   * @returns {void}
+   */
+  setDataFileConfig (dataFileConfig) {
+    if (this.dataFileConfig) {
+      createError({
+        error: new Error('Data file config is already set'),
+        message: 'Data file config is already set',
+        source: `WorkloadManager[${this.workloadId}]`,
+        severity: CONSTANTS.SEVERITY.ERROR,
+        subsystem: 'WORKLOAD_MANAGER',
+        broadcast: false
+      });
+    }
+
+    validateDataFileConfiguration(dataFileConfig);
+
+    this.dataFileConfig = dataFileConfig;
+  }
+
+  /**
+   * Returns the data for the sequence based on the distribution type.
+   *
+   * Called via common invocation system from the Sequence.
+   *
+   * @param {number} [VUIndex] - Index of the sequence.
+   *
+   * @returns {VUData} - Data for the sequence or scenario.
+   */
+  getDataForScenario = (VUIndex) => {
+    if (!this.dataFileConfig) {
+      return null;
+    }
+
+    if (!Array.isArray(this.dataFileConfig.contents) || !this.dataFileConfig.contents.length) {
+      return null;
+    }
+
+    let sliceIndices = [];
+
+    const { distributionType, contents } = this.dataFileConfig;
+
+    if (distributionType === DATAFILE_DISTRIBUTION_TYPES.ORDERED && VUIndex < contents.length) {
+      sliceIndices = [VUIndex];
+    }
+    else if (distributionType === DATAFILE_DISTRIBUTION_TYPES.RANDOM) {
+      // Pick random indexes from the data file contents array
+      sliceIndices = _.sampleSize(_.range(contents.length), DATAFILE_RANDOM_DISTRIBUTION.SAMPLE_SIZE);
+    }
+
+    return {
+      // Create an array of objects from contents based on the sliceIndices
+      data: sliceIndices.map((rowIndex) =>
+        Object.entries(contents[rowIndex]).map(([key, value]) => ({ key, value }))
+      ),
+      keys: Object.keys(contents[0]),
+      vuId: VUIndex,
+      sliceIndices,
+      randomize: distributionType === DATAFILE_DISTRIBUTION_TYPES.RANDOM
+    };
+  }
 };
+
+/**
+ * Validates the data file configuration.
+ *
+ * @param {object} dataFileConfig - Data file configuration.
+ *
+ * @throws {Error} - If data file config is not valid.
+ *
+ * @returns {void}
+ */
+function validateDataFileConfiguration (dataFileConfig) {
+  let error;
+
+  if (!dataFileConfig) {
+    error = new Error('Data file config is not provided');
+  }
+  else if (!dataFileConfig.distributionType) {
+    error = new Error('Data file distribution type is not provided');
+  }
+  else if (!dataFileConfig.contents) {
+    error = new Error('Data file contents are not provided');
+  }
+  else if (!Object.values(DATAFILE_DISTRIBUTION_TYPES).includes(dataFileConfig.distributionType)) {
+    error = new Error(`Data file distribution type should be either ${Object.values(DATAFILE_DISTRIBUTION_TYPES).join(' or ')}`);
+  }
+  else if (!Array.isArray(dataFileConfig.contents) || !dataFileConfig.contents.length) {
+    error = new Error('Data file contents should be a non-empty array');
+  }
+
+  if (error) {
+    createError({
+      error,
+      message: error.message,
+      source: `WorkloadManager[${this.workloadId}]`,
+      severity: CONSTANTS.SEVERITY.ERROR,
+      subsystem: 'WORKLOAD_MANAGER',
+      broadcast: false
+    });
+  }
+}
+
