@@ -14,7 +14,7 @@ var HttpProxy = require('http-proxy'),
   lruCache = require('lru-cache'),
   certMap = {},
   proxyConstants = require('../constants/ProxyConstants'),
-  proxyCertificateService = require('./proxyCertificateService'),
+  proxyCertificateService = require('./proxyCertificateService').proxyCertificateService,
   which = require('which'),
   { generateCert } = require('./proxyCertificateUtil'),
   certMapFile;
@@ -59,7 +59,6 @@ function getHostPortFromString (hostString, defaultPort) {
 }
 
 /**
- *
  * Exchanges data between proxy socket and request socket
  *
  * @param {*} socketRequest the Socket for the current request
@@ -105,7 +104,6 @@ function proxyData (socketRequest, bodyHead, httpVersion, freePort) {
 *
 * @param {number} freePort the free port on which https server is listening
 * @param {object} keys object that has the client key and certificate associated with the commonName
-* @param {httpProxyServer} proxyHttps the server used to proxy the requests
 * @param {function} capturedRequestCallback the callback function called for each request captured
 */
 function createHttpsServer (freePort, keys, capturedRequestCallback) {
@@ -136,7 +134,7 @@ function createHttpsServer (freePort, keys, capturedRequestCallback) {
               timeTaken = Date.now() - startTime;
             }
           }
-          capturedRequestCallback(req.url, req.method, req.headers, queryData, response, timeTaken);
+          capturedRequestCallback(req.url, req.method, req.headers, queryData, response, timeTaken, startTime);
           cleanUpResource(key);
           return responseBody;
         });
@@ -149,7 +147,7 @@ function createHttpsServer (freePort, keys, capturedRequestCallback) {
             }
           }
         delete response['body'];
-        capturedRequestCallback(req.url, req.method, req.headers, queryData, response, timeTaken);
+        capturedRequestCallback(req.url, req.method, req.headers, queryData, response, timeTaken, startTime);
         cleanUpResource(key);
       }
     })
@@ -215,11 +213,21 @@ function createHttpsServer (freePort, keys, capturedRequestCallback) {
 }
 
 /**
+ * Checks if OpenSSL is installed on the system.
  *
- * @returns {Promise} - resolves to true if openssl is present; false otherwise - does NOT reject
+ * @returns {Promise} - resolves to true if openssl is present or os is windows or mac; false otherwise - does NOT reject
  */
 function checkOpenSSLInstallation () {
   return new Promise((resolve) => {
+
+    /**
+     * This check is only required for linux as for other operating systems openssl
+     * is not a dependency, since we use node-forge to generate the ssl certificates.
+     *
+     * Hence we return true for operating systems other than linux
+     */
+    if (process.platform !== 'linux') return resolve(true);
+
     which('openssl', function (error) {
       if (error) {
         return resolve(false);
@@ -249,7 +257,6 @@ exports.electronProxy = {
   lastHttpsUrl: null,
 
   /**
-   *
    * @param {Number} port - Number denoting the port on which to start the proxy
    * @param {Function} proxyClosedCallback callback function called when the proxy closes
    * @param {Function} capturedRequestCallback callback function to capture the request
@@ -281,7 +288,8 @@ exports.electronProxy = {
     }
 
     try {
-      rootCA = await proxyCertificateService.generateRootCAForProxy(STORE_LOC);
+      const result = await proxyCertificateService.generateRootCAForProxy(STORE_LOC);
+      rootCA = result.rootCA;
       certMap = JSON.parse(fs.readFileSync(certMapFile, 'utf8'));
       certificatesGenerated = true;
     } catch (e) {
@@ -353,7 +361,7 @@ exports.electronProxy = {
             accumulateResponse(res, proxyRes, function (responseBody) {
               response['body'] = responseBody;
               timeTaken = Date.now() - start;
-              capturedRequestCallback(req.url, req.method, req.headers, queryData, response, timeTaken);
+              capturedRequestCallback(req.url, req.method, req.headers, queryData, response, timeTaken, start);
               return responseBody;
             });
           });
