@@ -1,23 +1,25 @@
 const _ = require('lodash'),
+  { isString } = require('../utils/utilities'),
   partitionService = require('./partitionService'),
   shellController = require('./shellController'),
-  partitionMigrationService = require('./partitionMigrationService'),
+  PartitionMigrationService = require('./partitionMigrationService'),
   partitionUtils = require('./partitionUtils'),
   shellMeta = require('./shellMeta');
 
 const namespace = 'users',
   scratchpadContext = { namespace: 'scratchPad', userId: 0, teamId: 0 },
-  { session } = require('electron');
+  { session } = require('electron'),
+  regionsMetaNamespace = 'regionsMeta';
 
 
 /**
 * Wipes out the associated cookie partition's storage.
-* @param {UUID} id, the partition id  whose associated cookie session needs to cleaned.
+* @param {string} id the partition id whose associated cookie session needs to cleaned.
 * @returns {Promise<String>}
 */
 function cleanCookiePartitionAssociated (id) {
   // Bail out if the id is either empty or not a string
-  if (_.isEmpty(id) || !_.isString(id)) {
+  if (!isString(id) || id === '') {
     return Promise.reject(new Error('Partition id is not valid string'));
   }
 
@@ -32,7 +34,7 @@ function cleanCookiePartitionAssociated (id) {
 
 /**
  * Gets the list of logged in user context partitions on v8 partition system
- * @returns {Object[]} Array of logged in user context partitions on v8 partition system
+ * @returns {Promise<Object[]>} Array of logged in user context partitions on v8 partition system
  */
 async function getAllV8UserContextPartitions () {
   const context = { namespace },
@@ -49,7 +51,8 @@ async function getAllV8UserContextPartitions () {
       username: partition.meta && partition.meta.username,
       profile_pic_url: partition.meta && partition.meta.profile_pic_url,
       lastUpdated: partition.meta && partition.meta.lastUpdated,
-      auth: partition.meta && partition.meta.auth
+      auth: partition.meta && partition.meta.auth,
+      region: partition.context && partition.context.region
     });
   });
 
@@ -58,7 +61,7 @@ async function getAllV8UserContextPartitions () {
 
 /**
  * Gets the list of logged in user context partitions on v7 partition system
- * @returns {Object[]} Array of logged in user context partitions on v7 partition system
+ * @returns {Promise<Object[]>} Array of logged in user context partitions on v7 partition system
  */
 async function getAllV7UserContextPartitions () {
   const v7UserContextPartitions = [],
@@ -82,7 +85,7 @@ async function getAllV7UserContextPartitions () {
 
 /**
  * Gets the least recently used v8 user context partition
- * @returns {String | undefined} Returns the partition id by comparing the v8 partition's last accessed timestamp
+ * @returns {Promise<String | undefined>} Returns the partition id by comparing the v8 partition's last accessed timestamp
  */
 async function getRecentV8UserPartition () {
   const partitions = await getAllV8UserContextPartitions();
@@ -102,7 +105,7 @@ async function getRecentV8UserPartition () {
 }
 
 /**
- * @returns {String} The partition id of v7 last active user if it is not yet migrated
+ * @returns {Promise<String>} The partition id of v7 last active user if it is not yet migrated
  */
 async function getV7ActiveUserPartitionIfNotYetMigratedToV8 () {
   const recentV7UserId = shellMeta.getLastActiveUser();
@@ -128,8 +131,9 @@ async function getV7ActiveUserPartitionIfNotYetMigratedToV8 () {
 
 const userPartitionService = {
   async init () {
-    await partitionMigrationService.init();
-    await partitionMigrationService.runV8PartitionsIntegrity();
+    const pms = PartitionMigrationService.instance();
+    await pms.init();
+    await pms.runV8PartitionsIntegrity();
     await this._rectifyTheActivePartitionOfApplication();
 
     /**
@@ -140,7 +144,7 @@ const userPartitionService = {
      * (v7 app does not self-correct this state and the user ends up being on signed-out exp.
      * We are making sure that even that partition is migrated as scratchpad :) )
      */
-    await partitionMigrationService.migrateSignedOutPartitionAsScratchpad();
+    await pms.migrateSignedOutPartitionAsScratchpad();
     pm.logger.info('userPartitionService~init: Success');
   },
 
@@ -184,17 +188,18 @@ const userPartitionService = {
 
   /**
    * Checks if a partitionId is associated with a user on v7 system
-   * @param {String} partitionId the partiiton id to check
-   * @returns {Boolean}
+   * @param {String} partitionId the partition id to check
+   * @returns {Promise<Boolean>}
    */
   isV7UserContextPartitionId (partitionId) {
-    return partitionMigrationService.isV7UserContextPartition(partitionId);
+    const pms = PartitionMigrationService.instance();
+    return pms.isV7UserContextPartition(partitionId);
   },
 
   /**
    * Checks if a partitionId is associated with a user on v8 system
-   * @param {String} partitionId the partiiton id to check
-   * @returns {Boolean}
+   * @param {String} partitionId the partition id to check
+   * @returns {Promise<Boolean>}
    */
   async isV8UserContextPartitionId (partitionId) {
     if (!partitionId) {
@@ -213,7 +218,7 @@ const userPartitionService = {
   /**
    * Gets the active partition used currently.
    * This includes the scratchpad partition as well.
-   * @returns {String | undefined} Returns partition id. undefined, if there are no user partitions.
+   * @returns {Promise<String | undefined>} Returns partition id. undefined, if there are no user partitions.
    */
   async getActivePartition () {
     const namespaceMeta = await partitionService.getNamespaceMeta(namespace);
@@ -224,7 +229,7 @@ const userPartitionService = {
   /**
    * Saves the current partition in use, only if partition is associated with a user context
    * @param {String} partitionId The partition id to be saved
-   * @returns {void} Does not return anything
+   * @returns {Promise<void>} Does not return anything
    */
   async setActivePartition (partitionId) {
     const namespaceMeta = await partitionService.getNamespaceMeta(namespace) || {};
@@ -233,12 +238,12 @@ const userPartitionService = {
     await partitionService.updateNamespaceMeta(namespace, namespaceMeta);
   },
 
-  // /**
-  //  * Finds the parititon id associated with the provided user information in v7 and v8.
-  //  * Returns the recently used partition, if there are multiple.
-  //  * @param {Object} userData The object containing the user information including userId
-  //  * @returns {String | undefined} The partition id associated, if available. undefined, otherwise.
-  //  */
+  /**
+   * Finds the partition id associated with the provided user information in v7 and v8.
+   * Returns the recently used partition, if there are multiple.
+   * @param {Object} userData The object containing the user information including userId
+   * @returns {Promise<String | undefined>} The partition id associated, if available. undefined, otherwise.
+   */
   async getPartitionForUser (userData) {
     // Bail out, if user id and team id are not available
     if (_.isEmpty(userData) || _.isEmpty(userData.id)) {
@@ -296,7 +301,7 @@ const userPartitionService = {
   /**
    * Finds the partition id associated with the provided user information.
    * @param {Object} userData The object containing the user information including userId
-   * @returns {String | undefined} The partition id associated, if available. undefined, otherwise.
+   * @returns {Promise<{ partitionId: string, isV8Partition: boolean } | undefined>} The partition id associated, if available. undefined, otherwise.
    */
   async getPartitionForUserContext (userData) {
     // Bail out, if user id and team id are not available
@@ -311,7 +316,8 @@ const userPartitionService = {
       context = {
         namespace,
         userId: userData.id,
-        teamId: userData.teamId
+        teamId: userData.teamId,
+        region: userData?.region
       },
       partitions = await partitionService.find(context);
 
@@ -350,7 +356,7 @@ const userPartitionService = {
    * Associate a partition with the provided user data.
    * @param {String} partitionId The partition id
    * @param {Object} userData The Object containing userId and teamId
-   * @returns {void}
+   * @returns {Promise<void>}
    */
   async associatePartitionToUserData (partitionId, userData) {
     if (!partitionId) {
@@ -358,14 +364,15 @@ const userPartitionService = {
     }
 
     // Bail out, if user id and team id are not available
-    if (_.isEmpty(userData) || !_.isString(userData.id) || !_.isString(userData.teamId) || userData.id === '0') {
+    if (!isString(userData?.id) || !isString(userData?.teamId) || userData?.id === '0') {
       return;
     }
 
     const context = {
         namespace,
         userId: userData.id,
-        teamId: userData.teamId
+        teamId: userData.teamId,
+        region: userData?.region
       },
       meta = {
         lastUpdated: new Date(),
@@ -373,19 +380,30 @@ const userPartitionService = {
         email: userData.email,
         username: userData.username,
         profile_pic_url: userData.profile_pic_url,
-        auth: userData.auth
+        auth: userData.auth,
+
+        // Used by Web Platform,
+        // We are saving the JSON string for two reasons:
+        // 1. partitionService will not save and return nested objects such as userData.auth
+        // 2. we retain the same structure as being used in Monolith, so no data transformation is needed
+        //    and we also get new fields which might be added to the object in future (such as `locale`)
+        raw: JSON.stringify(userData),
       };
 
     pm.logger.info('userPartitionService~associateTransitPartition: ', { partitionId, context });
     await partitionService.makeRawPartition(partitionId, context, meta);
+
+    // Set multi-login token in the meta.
+    await this.setMultiLoginToken(userData?.region, userData?.auth?.multi_login_token);
+
     this.logV8PartitionSystemState();
   },
 
   /**
-  //  * Creates a partition for the provided user data.
-  //  * @param {Object} userData The Object containing userId and teamId
-  //  * @returns {Promise<String>} Promise resolving to the partition id.
-  //  */
+   * Creates a partition for the provided user data.
+   * @param {Object} userData The Object containing userId and teamId
+   * @returns {Promise<String>} Promise resolving to the partition id.
+   */
   async associateTransitPartition (userData) {
     const partitionId = await this.getNewPartitionId();
 
@@ -403,13 +421,14 @@ const userPartitionService = {
       teamId: 0
     };
 
+    pm.logger.info('userPartitionService~getNewPartitionId: creating a partition with userId and teamId as 0');
     const partition = await partitionService.create(context, { lastUpdated: new Date() });
     return partition.id;
   },
 
   /**
    * Gets the list of logged in user context partitions
-   * @returns {Object[]} Array of logged in user context partitions
+   * @returns {Promise<Object[]>} Array of logged in user context partitions
    */
   async getAllUserContextPartitions () {
     const userContextPartitions = [];
@@ -418,7 +437,7 @@ const userPartitionService = {
   },
 
   /**
-   * @returns {Object[]} Array of distinct user accounts
+   * @returns {Promise<Object[]>} Array of distinct user accounts
    */
   async getAllUsers () {
     let userIdMap = {},
@@ -428,7 +447,15 @@ const userPartitionService = {
     allContextPartitions.forEach((partition) => {
       if (!userIdMap[partition.id]) {
         userIdMap[partition.id] = partition;
-        users.push({ id: partition.id, name: partition.name, email: partition.email, username: partition.username, profile_pic_url: partition.profile_pic_url });
+
+        users.push({
+          id: partition.id,
+          name: partition.name,
+          email: partition.email,
+          username: partition.username,
+          profile_pic_url: partition.profile_pic_url,
+          region: partition.region
+        });
       }
     });
 
@@ -437,7 +464,7 @@ const userPartitionService = {
 
   /**
    * Gets the least recently used partition across v7 and v8
-   * @returns {String|undefined} Returns the partition id by comparing the partition's last accessed timestamp
+   * @returns {Promise<String|undefined>} Returns the partition id by comparing the partition's last accessed timestamp
    */
   async getRecentPartition () {
     const partitions = await this.getAllUserContextPartitions();
@@ -494,8 +521,11 @@ const userPartitionService = {
   /**
    * Get user and team ID details for active partition
    *
-   * @returns {Object} { userId, teamId } details from active partition
-   * or undefined if partition / context not found
+   * @returns {Promise<{
+   *    userId: string;
+   *    teamId: string;
+   *    region: 'eu' | 'us'
+   * }>} User details from active partition or undefined if partition / context not found
    */
   async getUserContextForActivePartition () {
     const activePartitionId = await this.getActivePartition(),
@@ -506,9 +536,42 @@ const userPartitionService = {
       return;
     }
 
-    const { userId, teamId } = activePartition.context;
+    const { userId, teamId, region } = activePartition.context ?? {};
+    return { userId, teamId, region };
+  },
 
-    return { userId, teamId };
+  /**
+   * Stores the multi login token in the metadata for a specific region.
+   *
+   * @param {string} region - The region for which the multi login token will be stored.
+   * @param {string} multiLoginToken - The multi login token to be stored.
+   */
+  async setMultiLoginToken (region, multiLoginToken) {
+    if (!multiLoginToken || !region) {
+      return;
+    }
+
+    const regionsMetaData = await partitionService.getNamespaceMeta(regionsMetaNamespace) || {};
+
+    _.set(regionsMetaData, `${region}.multiLoginToken`, multiLoginToken);
+    await partitionService.updateNamespaceMeta(regionsMetaNamespace, regionsMetaData);
+  },
+
+  /**
+   * Retrieves the multi login token associated with the specified region.
+   *
+   * @param {string} region - The region for which to retrieve the multi login token.
+   * @returns {Promise<string>} - The multi login token associated with the region.
+   * @throws {Error} - If an error occurs while retrieving the multi login token.
+   */
+  async getMultiLoginToken (region) {
+    if (!region) {
+      return;
+    }
+
+    const namespaceMeta = await partitionService.getNamespaceMeta(regionsMetaNamespace) || {};
+
+    return _.get(namespaceMeta, `${region}.multiLoginToken`);
   }
 };
 
