@@ -6,7 +6,6 @@ const { performance } = require('perf_hooks');
 
 // third-party dependencies
 const electron = require('electron');
-const Storage = require('electron-json-storage');
 const _ = require('lodash').noConflict();
 const uuidV4 = require('uuid/v4');
 
@@ -20,12 +19,12 @@ const {
   DEFAULT_CONSOLE_BOUNDS,
   MIN_ALLOWED_WINDOW_HEIGHT,
   MIN_ALLOWED_WINDOW_WIDTH,
-  MAC_TRAFFIC_LIGHTS_POSITIONS_DEFAULT
+  MIN_ALLOWED_REQUESTER_WIDTH,
+  MAC_TRAFFIC_LIGHT_POSITION_DEFAULT
 } = require('../constants/WindowConstants');
 const { getValue } = require('../utils/processArg');
 const { AppLaunchPerfService } = require('./AppLaunchPerfService');
 const devUrls = require('../common/constants/dev-urls.js');
-const { createEvent } = require('../common/model-event');
 const getQPString = require('../utils/getQPString');
 const getAppUrl = require('../services/AppUrlService');
 const { getConfig } = require('./AppConfigService');
@@ -34,11 +33,16 @@ const partitionUtils = require('./partitionUtils');
 const { handlePartitioning, getActiveUser } = require('./userPartitionManager');
 const userPartitionService = require('./userPartitionService');
 const scratchPadPartitionService = require('./scratchPadPartitionService');
+const {
+  clearStatesPerWindow,
+  store,
+  STORE_KEYS
+} = require('../constants/store');
+const i18nService = require('./i18n/i18nService');
 
 // constants
-const { app, shell, webContents, session, nativeImage, nativeTheme } = electron;
+const { app, shell, session, nativeImage, nativeTheme } = electron;
 const BrowserWindow = electron.BrowserWindow;
-const SHELL_PARTITION_NAME = 'persist:postman_shell';
 const MAX_WINDOW_RESTORE_COUNT = 4;
 const ALL_DEV_TOOLS = ['requester', 'console'];
 const devtools = getValue('dev-tools') === true ? ALL_DEV_TOOLS : getValue('dev-tools').split(',');
@@ -49,14 +53,21 @@ const {
   HTML_MAP,
   IPC_EVENT_NAMES,
   AUTH_ACTIONS,
-  WINDOW_EVENTS_MAP,
   EVENT_BUS_EV_NAMES
 } = require('../constants');
+const regionService = require('./region.service.js');
 
-// See this discussion on the invisible frame around Windows 10/11 windows.
-// NOTE: This value is not affected by display scaling in testing.
-// NOTE: When a window is maximized, it has x,y of -8,-8 (we don't care about position when maximized in this code)
+// See this discussion on the invisible frame around Windows 10/11 windows:
 // https://stackoverflow.com/questions/34139450/getwindowrect-returns-a-size-including-invisible-borders
+// This happens because Electron is using the WS_THICKFRAME window style for resizable windows.
+// On earlier versions of Windows, thick frames were actually rendered as such, but by default, they're not anymore.
+// Probably to avoid breaking legacy apps, APIs that are not DWM-aware still behave as if they were rendered.
+// To compensate for this, we simply draw the window outside the screen by the width of that invisible border.
+// Electron does not provide the means to query the system for values like that so we hard code the most common value.
+// The frame width is 7 on most machines, though it can occasionally be 8 (in scaled multiscreen scenarios, for example).
+// Beware: this value is not to be confused with the location reported for maximized windows, which is mostly at x = -8, y = -8.
+// Confusingly enough, both of those coordinates can turn out to be -7 because of rounding on scaled screens.
+// To tell the scenarios apart, look at the value of y which is going to be either 0 (invisible frame) or at least -7 (maximized).
 const WIN_10_11_FRAME_WIDTH = 7;
 
 const TOGGLE_WINDOW_MAXIMIZE_EVENT = 'toggle-maximize',
@@ -68,7 +79,7 @@ const APP_BASE_PATH = url.format({ protocol: 'file', pathname: `${app.getAppPath
 const FILE_PROTOCOL = 'file:';
 
 let isWindows10or11 = false;
-if (process.platform === 'win32') {
+if (isWindows()) {
   const osReleaseMatch = (/(\d{2})\..*/).exec(os.release());
 
   if (osReleaseMatch) {
@@ -77,7 +88,7 @@ if (process.platform === 'win32') {
 }
 
 /**
- * Checks if the OS is Mac
+ * Checks if the OS is macOS.
  * @returns {Boolean}
  */
 function isDarwin () {
@@ -85,7 +96,7 @@ function isDarwin () {
 }
 
 /**
- * Checks if the OS is Mac
+ * Checks if the OS is Windows.
  * @returns {Boolean}
  */
 function isWindows () {
@@ -185,6 +196,7 @@ const windowManager = {
   windowState: {},
   isWindowVisibleByDefault: true,
   isFirstRequesterBooted: false,
+  hasWebPlatformNavigatedToMonolith: false,
 
   // #endregion
 
@@ -202,16 +214,6 @@ const windowManager = {
       this.webProxyPath = devUrls.WEB_PROXY;
     }
 
-    this.closedHandler = this.closedHandler.bind(this);
-    this.enterFullScreenHandler = this.enterFullScreenHandler.bind(this);
-    this.leaveFullScreenHandler = this.leaveFullScreenHandler.bind(this);
-    this.maximizeHandler = this.maximizeHandler.bind(this);
-    this.unmaximizeHandler = this.unmaximizeHandler.bind(this);
-    this.debouncedResizeHandler = _.debounce(this.resizeHandler.bind(this), 100);
-    this.debouncedStateChangeHandler = _.debounce(this.stateChangeHandler.bind(this), 100);
-    this.windowBoundsHandler = this.windowBoundsHandler.bind(this);
-    this.focusHandler = this.focusHandler.bind(this);
-    this.blurHandler = this.blurHandler.bind(this);
     this.channel = this.eventBus.channel('requester-window-events');
 
 
@@ -306,6 +308,16 @@ const windowManager = {
     this.isWindowVisibleByDefault = visible;
   },
 
+  isPureWebPlatformMode () {
+    // For use when serving from web platform and need to allow operations specific to the requester
+    // that are limited by isFirstRequesterBooted like closing, new requester window and new console window
+    const isRunningWebPlatform = getConfig('__WP_MODULE_FEDERATION__');
+
+    // The moment the user switches to a monolith route in any window, the control flow goes back to the default state.
+    const hasWPNavigatedToMonolithYet = this.hasWebPlatformNavigatedToMonolith;
+    return isRunningWebPlatform && !hasWPNavigatedToMonolithYet;
+  },
+
   /**
     * Show all windows.
   */
@@ -417,24 +429,6 @@ const windowManager = {
     }
   },
 
-  loadWindowState (windowName, callback) {
-    if (this.windowState[windowName]) {
-      return callback(this.windowState[windowName]);
-    }
-    else {
-      Storage.get(windowName, (error, lastWindowState) => {
-        if (error) { pm.logger.error('WindowManager~loadWindowState - Failed to load window state: ' + windowName); }
-        return callback(error || _.isEmpty(lastWindowState) ? this.getDefaultWindowState(windowName) : lastWindowState);
-      });
-    }
-  },
-
-  saveWindowState (windowName, callback) {
-    Storage.set(windowName, this.windowState[windowName], (error) => {
-      if (error) { pm.logger.error('WindowManager~loadWindowState - Failed to store window state: ' + windowName); }
-      return callback && callback();
-    });
-  },
 
   newRequesterOpened () {
     if (this.listenForRequesterWindow && this.initUrl && this.initWindowId) {
@@ -461,7 +455,7 @@ const windowManager = {
    * @typedef {{
    *  title: string;
    *  backgroundColor: string;
-   *  icon?: string;
+   *  icon?: Electron.NativeImage;
    *  webPreferences: {
    *    webSecurity?: boolean;
    *    backgroundThrottling?: boolean;
@@ -520,8 +514,8 @@ const windowManager = {
     // to go back to 0, 0 unexpectedly. This logic also allows a window that only has
     // a 1 pixel edge visible on the display to be considered 'visible', which is
     // likely to cause user complaints because they can't actually see the window.
-    // Look at how Chromium acts and try to emulate that, or even find the logic
-    // in the codebase and use that as a basis for this behavior.
+    // You may want to try emulating Chromium's logic when fixing this:
+    // https://source.chromium.org/chromium/chromium/src/+/main:chrome/browser/ui/window_sizer/window_sizer.cc
     let isWindowVisible = (
       (bounds.x >= nearestDisplay.bounds.x && bounds.x < nearestDisplay.bounds.x + nearestDisplay.bounds.width) &&
       (bounds.y >= nearestDisplay.bounds.y && bounds.y < nearestDisplay.bounds.y + nearestDisplay.bounds.height)
@@ -556,7 +550,7 @@ const windowManager = {
     return this
       .getOpenWindows('requester')
       .then((allOpenRequesterWindows) => {
-        if (_.isEmpty(allOpenRequesterWindows)) {
+        if (allOpenRequesterWindows.length === 0) {
           // If no requester windows are open, there will be only one requester window in the Window table
           // Restore that.
           return WindowController
@@ -580,11 +574,11 @@ const windowManager = {
   /**
    * This method helps to reload last active user session, for the provided BrowserWindow
    *
-   * @param {BrowserWindow} browserWindow
+   * @param {Electron.BrowserWindow} browserWindow
    */
   async reloadBrowserWindow (browserWindow) {
     const partitionToLoad = await userPartitionService.getActivePartition(),
-      isLoggedOut = _.isEmpty(await userPartitionService.getAllUsers()),
+      isLoggedOut = (await userPartitionService.getAllUsers()).length === 0,
       rootPageType = isLoggedOut ? 'scratchpad' : 'requester';
 
     let defaultWindowOptions = this.getWindowPref(app.getName(), partitionToLoad);
@@ -601,12 +595,23 @@ const windowManager = {
   },
 
   async newRequesterWindow (window = {}, params = {}) {
-    const partitionToLoad = await handlePartitioning(params.authResponse),
-      isLoggedOut = _.isEmpty(await userPartitionService.getAllUsers()),
+    const [partitionToLoad, userContext] = await Promise.all([
+      handlePartitioning(params.authResponse),
+      userPartitionService.getUserContextForActivePartition()
+    ]);
+
+    let region = userContext?.region;
+    if (params.authResponse?.authData?.userData?.region) {
+      region = params.authResponse.authData.userData.region;
+    }
+
+    pm.logger.info(`Main:windowManager~newRequesterWindow: Fetch and update runtime config: ${region}`);
+    await regionService.fetchRegionConfigAndOverride(region);
+
+    const isLoggedOut = (await userPartitionService.getAllUsers()).length === 0,
       rootPageType = isLoggedOut ? 'scratchpad' : 'requester';
 
-    let startTime = Date.now(),
-      windowName = 'requester',
+    let windowName = 'requester',
       bounds = {
         x: _.get(window, 'position.x'),
         y: _.get(window, 'position.y'),
@@ -616,7 +621,7 @@ const windowManager = {
       sanitizedBounds = this.sanitizeBounds(bounds, windowName);
 
     // If we are trying to open a requester window (except first requester) before the first requester is booted, we bail
-    if (!params.isFirstRequester && !this.isFirstRequesterBooted) {
+    if (!this.isPureWebPlatformMode() && !params.isFirstRequester && !this.isFirstRequesterBooted) {
       pm.logger.warn('WindowManager~newRequesterWindow - Bailing requester window creation as first requester is not booted.');
       return;
     }
@@ -633,7 +638,7 @@ const windowManager = {
         }),
         center: !window.position,
         show: this.isWindowVisibleByDefault,
-        minWidth: MIN_ALLOWED_WINDOW_WIDTH,
+        minWidth: MIN_ALLOWED_REQUESTER_WIDTH,
         minHeight: MIN_ALLOWED_WINDOW_HEIGHT
       }
     );
@@ -643,7 +648,7 @@ const windowManager = {
       Object.assign(browserWindowOptions, {
         titleBarStyle: 'hidden',
         titleBarOverlay: true,
-        trafficLightPosition: MAC_TRAFFIC_LIGHTS_POSITIONS_DEFAULT
+        trafficLightPosition: MAC_TRAFFIC_LIGHT_POSITION_DEFAULT
       });
     }
 
@@ -663,6 +668,11 @@ const windowManager = {
       isFirstRequester: params.isFirstRequester || false
     });
 
+    // When user signs in, the locale is sent in the auth response
+    // check if locale exists in auth response and change the locale accordingly
+    const authResponseLocale = _.get(params, ['authResponse', 'authData', 'userData', 'locale']);
+    authResponseLocale && i18nService.changeLocale(authResponseLocale);
+
     performance.mark(PERF_MARKS.MAIN_PROCESS_BROWSER_OPEN_COMPLETE);
     performance.measure(PERF_MEASURES.MAIN_PROCESS_BROWSER_OPEN_TIME, PERF_MARKS.MAIN_PROCESS_APP_READY);
 
@@ -696,7 +706,7 @@ const windowManager = {
       // This issue is only seen when there is a hidden window launched and another window is opened on top of it. In our case shared window is the hidden window
       // Adding a workaround for the issue with electron v11 where the requester window appears blank while launching the app.
       // Taking the focus away from requester window fixes the problem. This workaround needs to be put in when the content is ready. Similar issue is reported here https://github.com/electron/electron/issues/27353
-      if (this.isWindowVisibleByDefault && process.platform === 'win32') {
+      if (this.isWindowVisibleByDefault && isWindows()) {
         mainWindow.showInactive();
         mainWindow.focus();
       }
@@ -806,8 +816,7 @@ const windowManager = {
 
   async newConsoleWindow (window = {}, params = {}) {
     const rootPageType = 'console';
-    let startTime = Date.now(),
-      windowName = 'console',
+    let windowName = 'console',
       bounds = {
         x: _.get(window, 'position.x'),
         y: _.get(window, 'position.y'),
@@ -816,7 +825,7 @@ const windowManager = {
       },
       sanitizedBounds = this.sanitizeBounds(bounds, windowName);
 
-    if (!this.isFirstRequesterBooted) {
+    if (!this.isPureWebPlatformMode() && !this.isFirstRequesterBooted) {
       pm.logger.warn('WindowManager~newConsoleWindow - Bailing requester window creation as first requester is not booted!');
       return;
     }
@@ -955,19 +964,29 @@ const windowManager = {
   },
 
   addListeners (activeWindow) {
-    activeWindow.on('close', this.closedHandler);
-    activeWindow.on('move', this.debouncedStateChangeHandler);
-    activeWindow.on('resize', this.debouncedStateChangeHandler);
-    activeWindow.on('restore', this.windowBoundsHandler);
-    activeWindow.on('focus', this.focusHandler);
+    if (activeWindow.boundEventHandlers) {
+      return;
+    }
+
+    activeWindow.boundEventHandlers = {};
+
+    activeWindow.boundEventHandlers.close = this._closedHandler.bind(this, activeWindow);
+    activeWindow.boundEventHandlers.move = _.debounce(this._stateChangeHandler.bind(this, activeWindow), 100);
+    activeWindow.boundEventHandlers.resize = activeWindow.boundEventHandlers.move;
+    activeWindow.boundEventHandlers.restore = this._windowBoundsHandler.bind(this, activeWindow);
+    activeWindow.boundEventHandlers.focus = this._focusHandler.bind(this, activeWindow);
 
     if (activeWindow.type === 'requester') {
-      activeWindow.on('enter-full-screen', this.enterFullScreenHandler);
-      activeWindow.on('leave-full-screen', this.leaveFullScreenHandler);
-      activeWindow.on('maximize', this.maximizeHandler);
-      activeWindow.on('unmaximize', this.unmaximizeHandler);
-      activeWindow.on('resize', this.debouncedResizeHandler);
-      activeWindow.on('blur', this.blurHandler);
+      activeWindow.boundEventHandlers['enter-full-screen'] = this._enterFullScreenHandler.bind(this, activeWindow);
+      activeWindow.boundEventHandlers['leave-full-screen'] = this._leaveFullScreenHandler.bind(this, activeWindow);
+      activeWindow.boundEventHandlers.maximize = this._maximizeHandler.bind(this, activeWindow);
+      activeWindow.boundEventHandlers.unmaximize = this._unmaximizeHandler.bind(this, activeWindow);
+      activeWindow.boundEventHandlers.resize = _.debounce(this._resizeHandler.bind(this, activeWindow), 100);
+      activeWindow.boundEventHandlers.blur = this._blurHandler.bind(this, activeWindow);
+    }
+
+    for (const key in activeWindow.boundEventHandlers) {
+      activeWindow.on(key, activeWindow.boundEventHandlers[key]);
     }
   },
 
@@ -988,17 +1007,14 @@ const windowManager = {
     };
   },
 
-  stateChangeHandler (e) {
-    const activeWindow = e.sender;
-
-    if (!activeWindow || activeWindow.isDestroyed()) {
+  _stateChangeHandler (activeWindow, e) {
+    if (activeWindow.isDestroyed()) {
       return Promise.resolve();
     }
 
     const bounds = getBounds(activeWindow);
 
-    this.updateWindowState(activeWindow.windowName, e.sender);
-    this.saveWindowState(activeWindow.windowName);
+    this.updateWindowState(activeWindow.windowName, activeWindow);
 
     return WindowController.get({ browserWindowId: activeWindow.id })
       .then((window) => {
@@ -1048,9 +1064,14 @@ const windowManager = {
     }
   },
 
-  closedHandler (e) {
-    // The first requester window should not be closed before the web view boots up
-    if (!this.isFirstRequesterBooted) {
+  // This is only here as an external component was calling an internal event handler directly.
+  // It should be removed when the caller gets refactored.
+  notifyOfWindowClose (window, event) {
+    this._closedHandler(window, event);
+  },
+
+  _closedHandler (window, e) {
+    if (!this.isPureWebPlatformMode() && !this.isFirstRequesterBooted) {
       e.preventDefault();
       return;
     }
@@ -1061,13 +1082,13 @@ const windowManager = {
       return;
     }
 
-    let window = e.sender,
-      windowId = window.id;
+    const windowId = window.id;
 
     pm.logger.info(`WindowManager~closeHandler - Closed Window (id: ${windowId} )`);
 
     this.removeListeners(window);
     this.removeWindowId(windowId);
+    clearStatesPerWindow(window);
 
     this.deleteWindowFromDB(window);
 
@@ -1077,9 +1098,7 @@ const windowManager = {
     });
   },
 
-  focusHandler (e) {
-    let window = _.get(e, 'sender', {});
-
+  _focusHandler (window, e) {
     menuManager.updateMenuItems(window.type);
 
     this.channel.publish({
@@ -1088,49 +1107,42 @@ const windowManager = {
     });
   },
 
-  blurHandler (e) {
-    let window = _.get(e, 'sender', {});
-
+  _blurHandler (window, e) {
     this.channel.publish({
       type: 'blur',
       windowId: window.id
     });
   },
 
-  resizeHandler (e) {
-    let window = _.get(e, 'sender', {});
+  _resizeHandler (window, e) {
     this.channel.publish({
       type: 'resize',
       windowId: window.id
     });
   },
 
-  enterFullScreenHandler (e) {
-    let window = _.get(e, 'sender', {});
+  _enterFullScreenHandler (window, e) {
     this.channel.publish({
       type: 'enter-full-screen',
       windowId: window.id
     });
   },
 
-  leaveFullScreenHandler (e) {
-    let window = _.get(e, 'sender', {});
+  _leaveFullScreenHandler (window, e) {
     this.channel.publish({
       type: 'leave-full-screen',
       windowId: window.id
     });
   },
 
-  maximizeHandler (e) {
-    let window = _.get(e, 'sender', {});
+  _maximizeHandler (window, e) {
     this.channel.publish({
       type: 'maximize',
       windowId: window.id
     });
   },
 
-  unmaximizeHandler (e) {
-    let window = _.get(e, 'sender', {});
+  _unmaximizeHandler (window, e) {
     this.channel.publish({
       type: 'unmaximize',
       windowId: window.id
@@ -1138,20 +1150,15 @@ const windowManager = {
   },
 
   removeListeners (activeWindow) {
-    activeWindow.removeListener('close', this.closedHandler);
-    activeWindow.removeListener('resize', this.debouncedStateChangeHandler);
-    activeWindow.removeListener('move', this.debouncedStateChangeHandler);
-    activeWindow.removeListener('restore', this.windowBoundsHandler);
-    activeWindow.removeListener('focus', this.focusHandler);
-
-    if (activeWindow.type === 'requester') {
-      activeWindow.removeListener('enter-full-screen', this.enterFullScreenHandler);
-      activeWindow.removeListener('leave-full-screen', this.leaveFullScreenHandler);
-      activeWindow.removeListener('maximize', this.maximizeHandler);
-      activeWindow.removeListener('unmaximize', this.unmaximizeHandler);
-      activeWindow.removeListener('resize', this.debouncedResizeHandler);
-      activeWindow.removeListener('blur', this.blurHandler);
+    if (!activeWindow.boundEventHandlers) {
+      return;
     }
+
+    for (const key in activeWindow.boundEventHandlers) {
+      activeWindow.removeListener(key, activeWindow.boundEventHandlers[key]);
+    }
+
+    delete activeWindow.boundEventHandlers;
   },
 
   getFirstRequesterWindow () {
@@ -1209,7 +1216,7 @@ const windowManager = {
   },
 
   hasOpenWindows () {
-    return !_.isEmpty(BrowserWindow.getAllWindows());
+    return BrowserWindow.getAllWindows().length > 0;
   },
 
   restoreWindows (options = {}) {
@@ -1229,7 +1236,7 @@ const windowManager = {
           windowsToRestoreIds = _.slice(allWindowIds, 0, MAX_WINDOW_RESTORE_COUNT),
           idsToDelete = _.differenceWith(allWindowIds, windowsToRestoreIds);
 
-        if (_.isEmpty(idsToDelete)) {
+        if (idsToDelete.length === 0) {
           return windowsToRestore;
         }
         return WindowController
@@ -1246,7 +1253,7 @@ const windowManager = {
           return !windowIdsToSkip.includes(window.id);
         });
 
-        if (!_.isEmpty(allWindows)) {
+        if (allWindows.length > 0) {
           _.each(allWindows, (window) => {
             switch (window.type) {
               case 'requester':
@@ -1300,9 +1307,14 @@ const windowManager = {
    *
    * Electron issue - https://github.com/electron/electron/issues/8664
    */
-  windowBoundsHandler (e) {
-    let window = e.sender;
-    if (!window) {
+  _windowBoundsHandler (window, e) {
+    // Earlier, unminimizing a previously maximized window did not generate a 'restore' event on Windows.
+    // After https://github.com/electron/electron/pull/35342, Electron does generate the 'restore' event.
+    // This caused a fair bit of mayhem for some users with multiple screens in certain screen configurations.
+    // This is because APIs that are not DWM-aware report a maximized window to be outside the screen boundaries.
+    // See the comment near the definition of WIN_10_11_FRAME_WIDTH for more context.
+    // This function is not aware of that at all, so we've added a check to restore the earlier behavior.
+    if (isWindows() && window.isMaximized()) {
       return;
     }
 
@@ -1341,10 +1353,10 @@ const windowManager = {
     let window;
     return this.getOpenWindows('requester')
       .then((allOpenRequesterWindows) => {
-        if (_.isEmpty(allOpenRequesterWindows)) {
+        if (allOpenRequesterWindows.length === 0) {
           return WindowController.getAll({ type: 'requester' })
             .then((allRequesterWindows) => {
-              if (!_.isEmpty(allRequesterWindows)) {
+              if (allRequesterWindows.length > 0) {
                 window = this.newRequesterWindow(allRequesterWindows[0]);
               }
               if (!window) {
@@ -1374,10 +1386,11 @@ const windowManager = {
   sanitizeBounds (bounds = {}, windowName) {
     let defaultBounds,
       sanitizedCoordinates;
-
+    let WINDOW_MIN_WIDTH = MIN_ALLOWED_WINDOW_WIDTH;
     switch (windowName) {
       case 'requester':
         defaultBounds = DEFAULT_REQUESTER_BOUNDS;
+        WINDOW_MIN_WIDTH = MIN_ALLOWED_REQUESTER_WIDTH;
         break;
       case 'console':
         defaultBounds = DEFAULT_CONSOLE_BOUNDS;
@@ -1390,7 +1403,7 @@ const windowManager = {
     // Addded this safe check for the issue: https://github.com/postmanlabs/postman-app-support/issues/6304
     if (!_.isInteger(bounds.width) ||
       !_.isInteger(bounds.height) ||
-      bounds.width < MIN_ALLOWED_WINDOW_WIDTH ||
+      bounds.width < WINDOW_MIN_WIDTH ||
       bounds.height < MIN_ALLOWED_WINDOW_HEIGHT) {
       return defaultBounds;
     }
@@ -1516,6 +1529,10 @@ const windowManager = {
     this.isFirstRequesterBooted = true;
   },
 
+  setHasWPNavigatedToMonolithRouteOnce () {
+    this.hasWebPlatformNavigatedToMonolith = true;
+  },
+
   openDevTools: (window) => {
     window.webContents.openDevTools({ mode: 'detach', activate: true });
   },
@@ -1568,7 +1585,7 @@ exports.windowManager = windowManager;
  *  devtools?: boolean;
  * }} options
  *
- * @return {BrowserWindowLaunchParams}
+ * @return {Promise<BrowserWindowLaunchParams>}
  */
 const getWindowLaunchParams = async (rootPage, options) => {
   const { window, authResponse, isFirstRequester, partitionId } = options;
@@ -1589,7 +1606,7 @@ const getWindowLaunchParams = async (rootPage, options) => {
     authConfig: authConfig,
     appPath: app.getAppPath(),
     type: rootPage,
-    devtools: _.includes(devtools, rootPage),
+    devtools: devtools.includes(rootPage),
     primaryId: windowManager.primaryId,
     allIds: windowManager.openWindowIds,
     partitionId: partitionId,
@@ -1597,10 +1614,24 @@ const getWindowLaunchParams = async (rootPage, options) => {
     isSwitchUser: isSwitchUser,
     preloadFile: url.pathToFileURL(path.resolve(app.getAppPath(), 'preload/desktop/index.js')).href,
     authResponse: authResponse,
-    activeUser: await getActiveUser()
+    activeUser: await getActiveUser(),
+    systemLocale: app.getLocale()
   };
 };
 
+/**
+ * @param {{ sender: { webContents: any }}} ev
+ * @param {string} url
+ */
+const reloadBrowserWindow = (ev, url) => {
+  const webContents = ev.sender;
+  if (webContents && url) {
+    const win = BrowserWindow.fromWebContents(webContents);
+    pm.logger.info(`Main~reloadBrowserWindow (To Url): ${url}`);
+    win.loadURL(url);
+  }
+};
+
 /**
  * Attaches Main process BrowserWindow to Renderer process
  * IPC handlers.
@@ -1674,10 +1705,103 @@ function attachRendererIPCHandlers () {
    */
   ipcMain.handle(IPC_EVENT_NAMES.GET_PARTITION_ID, () => userPartitionService.getActivePartition());
 
+
+  /**
+   * Check whether monolith is loading from Web Platform for the first time
+   * And set the flag to true for subsequent navigations/calls.
+   */
+  ipcMain.handle(IPC_EVENT_NAMES.SET_HAS_WEB_PLATFORM_NAVIGATED_TO_MONOLITH, () => {
+    const isFirstNavigationToMonolith = !windowManager.hasWebPlatformNavigatedToMonolith;
+    windowManager.setHasWPNavigatedToMonolithRouteOnce();
+    return isFirstNavigationToMonolith;
+  });
+
+  /**
+   * Handler to open Electron context menu on right click for WEB PLATFORM
+   */
+  ipcMain.handle(IPC_EVENT_NAMES.OPEN_NATIVE_MENU, async (event, payload) => {
+
+    function sendMenuResponse (resp) {
+      return event.sender.send('CLOSE_POPUP_MENU', resp);
+    }
+    function addCbToMenuItems (template) {
+      if (!Array.isArray(template)) return;
+      return template.map((item) => {
+        const clickID = item.click;
+        return {
+          ...item,
+          click: () => sendMenuResponse(clickID),
+          submenu: addCbToMenuItems(item.submenu)
+        };
+      });
+    }
+
+    if (payload && payload.menu) {
+        const menu = electron.Menu.buildFromTemplate(addCbToMenuItems(payload.menu));
+        menu.on('menu-will-close', () => {
+          // emit event on menu close
+          sendMenuResponse();
+        });
+        menu.popup({ window: BrowserWindow.fromWebContents(event.sender) });
+    }
+  });
+
   ipcMain.handle(IPC_EVENT_NAMES.GET_PLATFORM_LAUNCH_PERF_METRICS, async () => {
     pm.logger.info(`Main~${IPC_EVENT_NAMES.GET_PLATFORM_LAUNCH_PERF_METRICS}`);
     return AppLaunchPerfService.getAllMeasures();
   });
+  ipcMain.handle(IPC_EVENT_NAMES.SCRATCHPAD_THEME_ACTION,
+
+    /**
+     * @param {'get' | 'set'} type
+     * @param {string=} value
+     */
+    async (_, type, value) => {
+      pm.logger.info(`Main~${IPC_EVENT_NAMES.SCRATCHPAD_THEME_ACTION}:: action: ${type}, value: ${value}`);
+      switch (type) {
+        case 'get':
+          return store.get(STORE_KEYS.SCRATCHPAD_THEME);
+        case 'set':
+          store.set(STORE_KEYS.SCRATCHPAD_THEME, value);
+      }
+    }
+  );
+
+  /**
+   * Handles Page navigations from file:// -> http:// protocols
+   * for Postman App
+   */
+  ipcMain.handle(IPC_EVENT_NAMES.GET_REMOTE_INIT_PATH, async () => {
+    pm.logger.info(`Main~${IPC_EVENT_NAMES.GET_REMOTE_INIT_PATH}:: ${store.get(STORE_KEYS.REMOTE_INIT_PATH)}`);
+    return store.get(STORE_KEYS.REMOTE_INIT_PATH);
+  });
+  ipcMain.handle(IPC_EVENT_NAMES.NAVIGATE_TO_REMOTE_URL, async (ev, initPath) => {
+    store.set(STORE_KEYS.REMOTE_INIT_PATH, initPath);
+
+    /** Passing userId/teamId as strings is important */
+    const to = getAppUrl(HTML_MAP['requester'], { userId: '0', teamId: '0' });
+    pm.logger.info(`Main~${IPC_EVENT_NAMES.NAVIGATE_TO_REMOTE_URL}: ${initPath}, To Url: ${to}`);
+    reloadBrowserWindow(ev, to);
+  });
+  ipcMain.handle(IPC_EVENT_NAMES.NAVIGATE_TO_SCRATCHPAD, async (ev, initPath) => {
+    const webContents = ev.sender;
+    const scratchpadUrlMap = store.get(STORE_KEYS.BASE_SCRATCHPAD_URL_PATHS);
+
+    if (!webContents) { return; }
+
+    try {
+      const win = BrowserWindow.fromWebContents(webContents);
+      const basePath = scratchpadUrlMap.get(win.id);
+      if (basePath) {
+        const baseUrl = new url.URL(basePath);
+        initPath && baseUrl.searchParams.append('initialPath', initPath);
+        pm.logger.info(`Main~${IPC_EVENT_NAMES.NAVIGATE_TO_SCRATCHPAD} (To Url): ${baseUrl}`);
+        win.loadURL(baseUrl.toString());
+      }
+    } catch (e) {
+      pm.logger.error(`Error: Main~${IPC_EVENT_NAMES.NAVIGATE_TO_SCRATCHPAD} Failed to parse URL:`, e);
+    }
+  });
 
   /**
    * Retrieves and sends data for offline page rendering
@@ -1691,14 +1815,37 @@ function attachRendererIPCHandlers () {
     return { userName, email };
   });
 
+  /**
+   * Returns the userPartition, including access_token
+   * This is used by Web Platform which does not have access (yet) to indexedDb
+   * @param {object} user
+   * @param {string} user.id
+   */
+  ipcMain.handle(IPC_EVENT_NAMES.GET_USER_PARTITION, async (event, user) => {
+    const partitionId = await userPartitionService.getPartitionForUser(user);
+    const userPartition = await partitionService.findOne(partitionId);
+
+    pm.logger.info(`Main~${IPC_EVENT_NAMES.GET_USER_PARTITION}`);
+    return userPartition;
+  });
+
+  /**
+   * Navigates to an external url
+   * This is used by Web Platform which does not have access (yet) to electron APIs directly
+   * @param {string} url
+   */
+  ipcMain.handle(IPC_EVENT_NAMES.OPEN_EXTERNAL_URL, async (event, url) => {
+    windowManager.openCustomURL(url);
+  });
+
   /**
    * Retrieves and sends partition data for the userId passed to this RPC
    *
    * @param {any} _ event
    * @param {{ id: string }} user the user for which we are finding the partition for
    */
-  ipcMain.handle(IPC_EVENT_NAMES.CHECK_USERS_PARTITION, async (event, user) => {
-    const partitionData = await userPartitionService.getPartitionForUser(user.id);
+  ipcMain.handle(IPC_EVENT_NAMES.CHECK_USERS_PARTITION, async (_event, user) => {
+    const partitionData = await userPartitionService.getPartitionForUser(user);
 
     pm.logger.info(`Main~${IPC_EVENT_NAMES.CHECK_USERS_PARTITION}: `, partitionData);
     return partitionData;
@@ -1717,28 +1864,51 @@ async function attachWebcontentListeners ({
   launchParams = {},
   webPreferences
 }) {
-  window.webContents.on('crashed', handleCrash(launchParams));
+  window.webContents.on('render-process-gone', handleCrash(launchParams));
 
   const allUsers = await userPartitionService.getAllUsers();
 
-  if (launchParams.type === 'requester' && !_.isEmpty(allUsers)) {
-    window.webContents.on('did-fail-load', async (event, errorCode, errorDescription, validatedURL, isMainFrame) => {
+  window.webContents.on('did-fail-load', async (event, errorCode, errorDescription, validatedURL, isMainFrame) => {
+    let errorObj = {
+      isNavigationError: false,
+      errorUrl: validatedURL,
+      errorCode: errorCode,
+      isSignedOutUser: !allUsers || allUsers.length === 0
+    };
+    const eventName = 'did-fail-load';
+    pm.logger.info(`Main ~ ${eventName}: Couldn\'t load the app.`, errorObj, errorCode, errorDescription, isMainFrame);
+
+    // bailing out in case the did-fail-load did not triggered from main frame
+    // reference: https://postmanlabs.atlassian.net/browse/CUE-3617
+    if (!isMainFrame) {
+      return;
+    }
+
+    return getAbsolutePageURL('offline', launchParams, webPreferences, {
+      eventName,
+      errorObj
+    })
+    .then((url) => {
+      pm.logger.info(`Main ~ ${eventName}: Navigating to offline page.`, url);
+      window.loadURL(url);
+    })
+    .catch((e) => {
+      pm.logger.error('Error: did-fail-load: ', errorObj);
+    });
+  });
+
+  window.webContents.on('did-navigate', async (__, url, httpResponseCode) => {
+    if (httpResponseCode >= 400 && httpResponseCode <= 599) {
       let errorObj = {
-        isNavigationError: false,
-        errorUrl: validatedURL,
-        errorCode: errorCode
+        isNavigationError: true,
+        errorUrl: url,
+        errorCode: httpResponseCode,
+        isSignedOutUser: !allUsers || allUsers.length === 0
       };
-      const eventName = 'did-fail-load';
-      pm.logger.info(`Main ~ ${eventName}: Couldn\'t load the app.`, errorObj, errorCode, errorDescription, isMainFrame);
-
-      // bailing out in case the did-fail-load did not triggered from main frame
-      // reference: https://postmanlabs.atlassian.net/browse/CUE-3617
-      if (!isMainFrame) {
-        return;
-      }
-
-      return getAbsolutePageURL('offline', launchParams, webPreferences, {
-        eventName: 'did-fail-load',
+      const eventName = 'did-navigate';
+      pm.logger.info(`Main ~ ${eventName}: Couldn\'t load the app.`, errorObj);
+      getAbsolutePageURL('offline', launchParams, webPreferences, {
+        eventName,
         errorObj
       })
       .then((url) => {
@@ -1746,33 +1916,10 @@ async function attachWebcontentListeners ({
         window.loadURL(url);
       })
       .catch((e) => {
-        pm.logger.error('Error: did-fail-load: ', errorObj);
+        pm.logger.error('Error: did-navigate: ', errorObj);
       });
-    });
-
-    window.webContents.on('did-navigate', async (__, url, httpResponseCode) => {
-      if (httpResponseCode >= 400 && httpResponseCode <= 599) {
-        let errorObj = {
-          isNavigationError: true,
-          errorUrl: url,
-          errorCode: httpResponseCode
-        };
-        const eventName = 'did-navigate';
-        pm.logger.info(`Main ~ ${eventName}: Couldn\'t load the app.`, errorObj);
-        getAbsolutePageURL('offline', launchParams, webPreferences, {
-          eventName,
-          errorObj
-        })
-        .then((url) => {
-          pm.logger.info(`Main ~ ${eventName}: Navigating to offline page.`, url);
-          window.loadURL(url);
-        })
-        .catch((e) => {
-          pm.logger.error('Error: did-navigate: ', errorObj);
-        });
-      }
-    });
-  }
+    }
+  });
 
   if (['requester', 'scratchpad', 'console'].includes(launchParams.type)) {
     // This is to prevent transition to unwanted URLs in the app. We attach a listener
@@ -1813,40 +1960,6 @@ async function attachWebcontentListeners ({
   }
 }
 
-/**
- * @param {*} webContents Active BrowserWindow webContents
- * @param {BrowserWindowLaunchParams} launchParams Launch Params with which window was launched
- */
-function initializeView (webContents, launchParams) {
-  // Send updated launch params
-  webContents.send('electronWindowMessage', {
-    name: WINDOW_EVENTS_MAP[launchParams.type],
-    data: _.assign(
-      launchParams,
-      {
-        partitionId: launchParams.partitionId,
-        allIds: windowManager.openWindowIds,
-        primaryId: windowManager.primaryId
-      }
-    )
-  });
-
-  if (launchParams.type === 'runner') {
-    // Not sure if runner logic is required anymore??
-    pm.eventBus.channel('runner-events')
-      .publish(createEvent('set', 'runner', launchParams.testAttr));
-  }
-  else {
-    sendUsersDataToRenderer(webContents);
-  }
-
-  if (launchParams.isNewUser) {
-    const config = launchParams.authConfig;
-    pm.eventBus.channel('onboarding-events')
-      .publish(createEvent('onboard_user', 'onboarding', { config }));
-  }
-}
-
 /**
  * Send updated LoggedIn users data, when the event is triggered, once
  * at launch.
@@ -1884,9 +1997,7 @@ const getAbsolutePageURL = async (_type, launchParams, webPreferences, options =
     const { eventName, errorObj = {} } = options;
     pm.logger.info(`Main ~ ${eventName}: Couldn\'t load the app.`, errorObj);
     additionalParams = {
-      isNavigationError: errorObj.isNavigationError,
-      errorUrl: errorObj.errorUrl,
-      errorCode: errorObj.errorCode
+      ...errorObj
     };
   }
 
@@ -1913,13 +2024,19 @@ const getAbsolutePageURL = async (_type, launchParams, webPreferences, options =
   let basePath = BASE_URL;
   if (['scratchpad', 'offline'].includes(type)) {
     basePath = url.format({ protocol: 'file', pathname: `${app.getAppPath()}/html` });
+    const fileUrl = `${basePath}/${srcUrl}`;
 
-    return `${basePath}/${srcUrl}`;
+    if (type === 'scratchpad') {
+      const scratchpadUrlMap = store.get(STORE_KEYS.BASE_SCRATCHPAD_URL_PATHS);
+      scratchpadUrlMap.set(params.browserWindowId, fileUrl);
+    }
+
+    return fileUrl;
   }
 
   let absoluteSrcUrl = `${basePath}/${srcUrl}`;
 
-  const isLoggedOut = _.isEmpty(await userPartitionService.getAllUsers());
+  const isLoggedOut = (await userPartitionService.getAllUsers()).length === 0;
   if (!isLoggedOut) {
     const userContext = await userPartitionService.getUserContextForActivePartition();
     absoluteSrcUrl = getAppUrl(htmlFileName, userContext);
