@@ -3,7 +3,6 @@ const fs = require('fs'),
   path = require('path'),
 
   async = require('async'),
-  mkdirp = require('mkdirp'),
 
   homeDir = os.homedir(),
   defaultWorkingDir = path.join(homeDir, 'Postman', 'files'),
@@ -37,12 +36,7 @@ const fs = require('fs'),
 
       // Create the default working directory
       (next) => {
-        // Node version < 10 does't support creating directory recursively, hence using external
-        // library
-        mkdirp(defaultWorkingDir, next);
-
-        // @todo Use fs.mkdir when node version >=10
-        // fs.mkdir(defaultWorkingDir, { recursive: true }, next);
+        fs.mkdir(defaultWorkingDir, { recursive: true }, next);
       }
     ], cb);
   };
